#ifndef CUTE4HERA_H_
#define CUTE4HERA_H_

#include "CuTe_Options.h"


//Forward declare LHAPDF classes
namespace LHAPDF {
	class AlphaS;
	class PDF;
}  // namespace LHAPDF

namespace CuTe4HERA {

class _CuTe4HERA_Calculator;	//!< Forward declare the class, which handles the calculation

using CuTe::SOPS_CS;	//!< Class which contains all options: (S)et of (OP)tion (S)ets for CuTe

using std::string;

/** Class to store the results
 *	Matched = TF * RES + Rms^TF * ( FO - TF * SQT)
 */
class Results{
public:
	double	RES;		//!< pure resummed result
	double	RES_DInt;	//!< symmetric, absolute integration error for RES
	double	SQT;		//!< small-qT-expansion of fixed-order (FO)
	double	SQT_DInt;	//!< symmetric, absolute integration error for SQT
	double	TF;			//!< transition-factor
	double	Rms;		//!< matching-scheme factor. !! The power TF is already included => this->Rms = Rms^TF !!

	/** CUBA_FAIL indicates the status of the cuba integration
	 * 	==-2 : uninitialized
	 *  ==-1 : integral-dimension out of range
	 * 	== 0 : OK
	 *  ==+1 : accuracy goal not reached
	 *   >+1 : accuracy goal not reached, increase maxeval by approximately CUBA_FAIL
	 */
	int	CUBA_FAIL;

	inline Results()
	:RES(0.),RES_DInt(0.)
	,SQT(0.),SQT_DInt(0.)
	,TF(0.),Rms(0.)
	,CUBA_FAIL(-2){

	}

};


/** Interface to CuTe	*/
class Interface {
protected:
	_CuTe4HERA_Calculator *calc;
public:

	/** Standard constructor !! Not initialized !! */
	Interface();
	/** Construct and initialize with infile */
	Interface(const string& infilename);
	/** Construct and initialize with option-set */
	Interface(const SOPS_CS& OP);
	/** Copy-construct and initialize */
	Interface(const Interface& other);
	/** Copy and initialize */
	Interface& operator=(const Interface& other);

	/** (Re-)Initialize with current option-set.	*/
	void init();
	/** Re-initialize with new infile.	*/
	void init(const string& infilename);
	/** Re-initialize with new option-set.	*/
	void init(const SOPS_CS& OP);

	~Interface();

	/** Change currently used value of mu to a fixed value (for all qT)	 */
	void change_mu_to_value(const double mu);
	/** Change currently used value of mu and q*.
	* The scale mu will be dynamically calculated by mu(qT,q*) = qT + q* x exp( -qT/q* )	*/
	void change_qstar_to_value(const double qstar);
	/** Change currently used value of mu and q*.
	* The scale mu will be dynamically calculated by mu(qT,q*) = qT + q* x exp( -qT/q* )
	* "automatic" solves eta(mu==q*)==1 for q* which depends on alpha_s.	*/
	void change_qstar_to_automatic();
	/** Get the "automatic" value of q*	 */
	double get_qstar()const;

	/** Change currently used PDF and alpha_s routine.
	* Using the "automatic" q_star calculation, this can change the value of q_star, which depends on alpha_s.
	* CuTe will NOT destroy the objects behind the pointer.	*/
	void change_PDF(LHAPDF::PDF* lha_pdf, LHAPDF::AlphaS* lha_alphas);

	/** Change currently used Lambda_NP*/
	void change_NPE(const double Lambda_NP);
	/** Change currently used Lambda_NP and NPE-scheme.	*/
	void change_NPE(const double Lambda_NP, const CuTe::NS_enums::NS_NP::CUTOFF NP_scheme);

	/** Calculate double differential results for qT and y.
	* Return cuba integration status (see Results class)
	* and fill the Results object.	*/
	int calculate(const double qT, const double y, Results& results);

	/** Return the currently used option-set.	*/
	const SOPS_CS& get_options()const;

};

}  // namespace CuTe4HERA
#endif
