/******************************************************************************
**  hplog: a subroutine for the evaluation of harmonic polylogarithms
**         Version 1.0         12/07/2001
**  described in: 
**  T.Gehrmann and E.Remiddi: Numerical Evaluation of the Harmonic 
**                            Polylogarithms up to Weight 4
**                            (hep-ph/0107173; CERN-TH/2001/188)
**  the harmonic polylogarithms are defined in: 
**  E.Remiddi and J.Vermaseren: Harmonic Polylogarithms
**                            (hep-ph/9905237; Int.J.Mod.Phys. A15 (2000) 725)
**  email:
**  Thomas.Gehrmann@cern.ch and Ettore.Remiddi@bo.infn.it
**
******************************************************************************/

#include "hpl1d.hpp"

/**
 *
 * @param x is the argument of the 1dHPL's (1 dimensional Harmonic PolyLogarithms)
		to be evaluated;
 * @param nw is the maximum weight of the required 1dHPL's;
 * the maximum allowed value of nw of this implementation is 4;
 * @param Hc1,Hc2,Hc3,Hc4 are the complex*16 values of the 1dHPL;
 * they must all be supplied in the arguments even if some of them are not to be evaluated;
 * @param Hr1,Hr2,Hr3,Hr4 are the double precision real parts of Hc1,Hc2,Hc3,Hc4;
 * @param Hi1,Hi2,Hi3,Hi4 are the double precision real parts of Hc1,Hc2,Hc3,Hc4;
 * @param n1,n2 is the required range of indices, the allowed ranges are (0,1), (-1,0), (-1,1) ;
 */
void hpl::hplog(double x, int nw,
		FA1dC11& Hc1, FA2dC11& Hc2, FA3dC11& Hc3, FA4dC11& Hc4,
		FA1dD11& Hr1, FA2dD11& Hr2, FA3dD11& Hr3, FA4dD11& Hr4,
		FA1dD11& Hi1, FA2dD11& Hi2, FA3dD11& Hi3, FA4dD11& Hi4,
		int n1, int n2)
{

	using namespace details;
	int infilldim;
	FA1dI13 infill;
	const double r2   = 1.4142135623730950488;

	// check on the weight nw
	if ( (nw<1) || (nw>4) )
	{
		std::cerr << " illegal call of hpl::hplog with second argument\n"
			" (the weight) = " << nw << " the allowed values of the weight are 1,2,3,4";
	return;
	}

	// check on the range n1:n2
	if ( (n1==-1) && (n2==0) )
    {
        infilldim =  2;
        infill(1) =  0;
        infill(2) = -1;
    }
	else if ( (n1==0) && (n2==1) )
	{
        infilldim =  2;
        infill(1) =  0;
        infill(2) =  1;
	}
	else if ( (n1==-1) && (n2==1) )
	{
        infilldim =  3;
        infill(1) =  0;
        infill(2) = -1;
        infill(3) =  1;
	}
	else
	{
        std::cerr << " illegal call of hpl::hplog with the two last "
        		"arguments = (',n1,',',n2,')\n";
        std::cerr <<" the allowed values are (-1,0), (0,1), (-1,1) \n";
        return;
	}

	// setting the immaginary parts equal to zero
	setzero(nw,Hi1,Hi2,Hi3,Hi4,n1,n2);

	// looking at the range of the argument
	// r2 = sqrt(2.)
	double r2m1 = r2 - 1;
	double r2p1 = r2 + 1;
	if ( ( x>-r2m1 ) && ( x<=r2m1) )
	{
		//        print*, ' eval1dhpl:      x = ',x,', call eval1dhplat0 '
        eval1dhplat0(x,nw,Hc1,Hc2,Hc3,Hc4,Hr1,Hr2,Hr3,Hr4,Hi1,Hi2,Hi3,Hi4,n1,n2,
        		infilldim,infill);
	}
	else if ( x==1. )
	{
		//        print*, ' eval1dhpl:      x = ',x,', call eval1dhplin1 '
		eval1dhplin1(x,nw,Hc1,Hc2,Hc3,Hc4,Hr1,Hr2,Hr3,Hr4,Hi1,Hi2,Hi3,Hi4,n1,n2,
				infilldim,infill);
	}
	else if ( ( x>r2m1 ) && ( x<=r2p1) )
	{
		//        print*, ' eval1dhpl:      x = ',x,', call eval1dhplat1 '
		eval1dhplat1(x,nw,Hc1,Hc2,Hc3,Hc4,Hr1,Hr2,Hr3,Hr4,Hi1,Hi2,Hi3,Hi4,n1,n2,
				infilldim,infill);
	}
	else if ( ( x>r2p1 ) )
	{
		//        print*, ' eval1dhpl:      x = ',x,', call eval1dhplatinf '
        eval1dhplatinf(x,nw,Hc1,Hc2,Hc3,Hc4,Hr1,Hr2,Hr3,Hr4,Hi1,Hi2,Hi3,Hi4,n1,n2,
        		infilldim,infill);
      }
	else if ( ( x<=-r2p1) )
	{
		//        print*, ' eval1dhpl:      x = ',x,', call eval1dhplatminf '
        eval1dhplatminf(x,nw,Hc1,Hc2,Hc3,Hc4,Hr1,Hr2,Hr3,Hr4,Hi1,Hi2,Hi3,Hi4,n1,n2,
        		infilldim,infill);
	}
	else if ( x==-1. )
	{
		//        print*, ' eval1dhpl:      x = ',x,', call eval1dhplinm1 '
        eval1dhplinm1(x,nw,Hc1,Hc2,Hc3,Hc4,Hr1,Hr2,Hr3,Hr4,Hi1,Hi2,Hi3,Hi4,n1,n2,
        		infilldim,infill);
	}
	else if ( ( x>-r2p1 ) && ( x<=-r2m1) )
	{
		//        print*, ' eval1dhpl:      x = ',x,', call eval1dhplatm1 '
        eval1dhplatm1(x,nw,Hc1,Hc2,Hc3,Hc4,Hr1,Hr2,Hr3,Hr4,Hi1,Hi2,Hi3,Hi4,n1,n2,
        		infilldim,infill);
	}
}


/**
 *
 * @brief evaluates 1dhpl's in the 0-range  -(r2-1) < y <= (r2-1)
 * by direct series expansion (Bernoulli-accelerated)
 */
void hpl::details::eval1dhplat0(double y, int nw,
		FA1dC11& H1,  FA2dC11& H2,  FA3dC11& H3,  FA4dC11& H4,
		FA1dD11& HY1, FA2dD11& HY2, FA3dD11& HY3, FA4dD11& HY4,
		FA1dD11& Hi1, FA2dD11& Hi2, FA3dD11& Hi3, FA4dD11& Hi4,
		int n1, int n2, int infilldim, FA1dI13 infill)
{
	// evaluate the irreducible 1dHPL's first
	fillh1(y,H1,HY1,Hi1,n1,n2);
	if ( nw==1 ) return;
	fillirr1dhplat0(y,nw,HY1,HY2,HY3,HY4,n1,n2);
	// then the reducible 1dHPL's
	fillred1dhpl(nw,H1,H2,H3,H4,HY1,HY2,HY3,HY4,Hi1,Hi2,Hi3,Hi4,n1,n2,infilldim,infill);
	return;
}


/**
 *
 * @brief evaluates 1dhpl's for y=1 (explicit values are tabulated)
 */
void hpl::details::eval1dhplin1(double y, int nw,
		FA1dC11& H1,  FA2dC11& H2,  FA3dC11& H3,  FA4dC11& H4,
		FA1dD11& HY1, FA2dD11& HY2, FA3dD11& HY3, FA4dD11& HY4,
		FA1dD11& Hi1, FA2dD11& Hi2, FA3dD11& Hi3, FA4dD11& Hi4,
		int n1, int n2, int infilldim, FA1dI13 infill)
{
	// evaluate the irreducible 1dHPL's first
	fillh1(y,H1,HY1,Hi1,n1,n2);
	if ( nw==1 ) return;
	fillirr1dhplin1(y,nw,HY1,HY2,HY3,HY4,n1,n2);
	// then the reducible 1dHPL's
    fillred1dhpl(nw,H1,H2,H3,H4,HY1,HY2,HY3,HY4,Hi1,Hi2,Hi3,Hi4,n1,n2,infilldim,infill);
    if (n2==0) return;
    // correct the ill-defined entries
    HY2(1,0) = - HY2(0,1);
	Hi2(1,0) = 0.;
	H2(1,0) = dcmplx(HY2(1,0),Hi2(1,0)*pi);
	if ( nw==2 ) return;
	HY3(1,0,0) = HY3(0,0,1);
	Hi3(1,0,0) = 0.;
	H3(1,0,0) = dcmplx(HY3(1,0,0),Hi3(1,0,0)*pi);
	if ( nw==3 ) return;
	HY4(1,0,0,0) = -HY4(0,0,0,1);
	Hi4(1,0,0,0) = 0.;
	H4(1,0,0,0) = dcmplx(HY4(1,0,0,0),Hi4(1,0,0,0)*pi);
	return;
}


/**
 *
 * @brief evaluates 1dhpl's in the 1-range  (r2-1) < y <= (r2+1)
 * evaluating first the H(..,r=(1-y)/(1+y)) by calling eval1dhplat0(r)
 * and then expressing H(..,y=(1-r)/(1+r)) in terms of H(..,r)
 */
void hpl::details::eval1dhplat1(double y, int nw,
		FA1dC11& H1,  FA2dC11& H2,  FA3dC11& H3,  FA4dC11& H4,
		FA1dD11& HY1, FA2dD11& HY2, FA3dD11& HY3, FA4dD11& HY4,
		FA1dD11& Hi1, FA2dD11& Hi2, FA3dD11& Hi3, FA4dD11& Hi4,
		int n1, int n2, int infilldim, FA1dI13 infill)
{
	// additional arrays required within this routine
	FA1dD11 HR1;
	FA2dD11 HR2;
	FA3dD11 HR3;
	FA4dD11 HR4;

	// the nw = 1 case
	fillh1(y,H1,HY1,Hi1,n1,n2);
	if ( nw==1 ) return;
	// the nw > 1 case
	double r = (1.-y)/(1.+y);
	//      print*,' eval1dhplat1: y = ',y,', r = ',r
	// the whole (-1,1) range is in general needed for any pair (n1,n2)
    fillirr1dhplat0(r,nw,HR1,HR2,HR3,HR4,-1,1);
	// fillirr1dhplat1 takes care automatically of all the immaginary
	// parts as well as of the jump across y=1
	fillirr1dhplat1(r,nw,HR1,HR2,HR3,HR4,HY1,HY2,HY3,HY4,Hi1,Hi2,Hi3,Hi4,n1,n2);
	// then the reducible 1dHPL's
	fillred1dhpl(nw,H1,H2,H3,H4,HY1,HY2,HY3,HY4,Hi1,Hi2,Hi3,Hi4,n1,n2,infilldim,infill);
	return;
}


/**
 *
 * @brief evaluates 1dhpl's in the inf-range  (r2+1) < abs(y)
 * evaluating first the H(..,x=1/y) by calling eval1dhplat0(x)
 * and then expressing H(..,y=1/x) in terms of H(..,x)
 */
void hpl::details::eval1dhplatinf(double y, int nw,
		FA1dC11& H1,  FA2dC11& H2,  FA3dC11& H3,  FA4dC11& H4,
		FA1dD11& HY1, FA2dD11& HY2, FA3dD11& HY3, FA4dD11& HY4,
		FA1dD11& Hi1, FA2dD11& Hi2, FA3dD11& Hi3, FA4dD11& Hi4,
		int n1, int n2, int infilldim, FA1dI13 infill)
{
	// additional arrays required within this routine
	FA1dD11 HX1;
	FA2dD11 HX2;
	FA3dD11 HX3;
	FA4dD11 HX4;
	// the nw = 1 case
	fillh1(y,H1,HY1,Hi1,n1,n2);
	if ( nw==1 ) return;
	// the nw > 1 case
	double x = 1./y;
	//      print*,' eval1dhplatinf: y = ',y,', x = ',x
	fillirr1dhplat0(x,nw,HX1,HX2,HX3,HX4,n1,n2);
	// fillirr1dhplatinf takes care automatically of all the immaginary
	// parts as well as of the jump across y=1
	fillirr1dhplatinf(x,nw,HX1,HX2,HX3,HX4,HY1,HY2,HY3,HY4,Hi1,Hi2,Hi3,Hi4,n1,n2);
	// then the reducible 1dHPL's
	fillred1dhpl(nw,H1,H2,H3,H4,HY1,HY2,HY3,HY4,Hi1,Hi2,Hi3,Hi4,n1,n2,infilldim,infill);
	return;
}


/**
 *
 * @brief evaluates 1dhpl's for y=-1 (explicit values are tabulated)
 */
void hpl::details::eval1dhplinm1(double y, int nw,
		FA1dC11& H1,  FA2dC11& H2,  FA3dC11& H3,  FA4dC11& H4,
		FA1dD11& HY1, FA2dD11& HY2, FA3dD11& HY3, FA4dD11& HY4,
		FA1dD11& Hi1, FA2dD11& Hi2, FA3dD11& Hi3, FA4dD11& Hi4,
		int n1, int n2, int infilldim, FA1dI13 infill)
{
	// additional arrays required within this routine
	FA1dC11 G1;
	FA2dC11 G2;
	FA3dC11 G3;
	FA4dC11 G4;
	FA1dD11 GY1, Gi1;
	FA2dD11 GY2, Gi2;
	FA3dD11 GY3, Gi3;
	FA4dD11 GY4, Gi4;
	FA1dI13 istorfill;
	const int nphase_[3]={-1,1,-1};
	FA1dI11 nphase(nphase_);
	int nph1, nph2, nph3, nph4;
	// print*,' eval1dhplatm1: y = ',y
	if (infilldim==2)
	{
		for (int i=1; i<=2; ++i)
		{
			istorfill(i) = infill(i);
			infill(i) = -istorfill(i);
		}
	}
	// evaluate H(...,-y)
	setzero(nw,Gi1,Gi2,Gi3,Gi4,-n2,-n1);
	Gi1(0) = -1;
	eval1dhplin1(-y,nw,G1,G2,G3,G4,GY1,GY2,GY3,GY4,Gi1,Gi2,Gi3,Gi4,-n2,-n1,infilldim,infill);
	if (infilldim==2)
	{
         for (int i=1; i<=2; ++i)
         {
            infill(i) = istorfill(i);
         }
	}
	// fill the arrays H's
	for (int k1=n1; k1<=n2; ++k1)
	{
        nph1 = nphase(k1);
        HY1(k1) =   nph1*GY1(-k1);
        Hi1(k1) = - nph1*Gi1(-k1);
        H1(k1)  = dcmplx(HY1(k1),Hi1(k1)*pi);
        if ( nw>1 )
        {
        	for (int k2=n1; k2<=n2; ++k2)
        	{
				nph2 = nph1*nphase(k2);
				HY2(k1,k2) =   nph2*GY2(-k1,-k2);
				Hi2(k1,k2) = - nph2*Gi2(-k1,-k2);
				H2(k1,k2)  = dcmplx(HY2(k1,k2),Hi2(k1,k2)*pi);
				if ( nw>2 )
				{
					for (int k3=n1; k3<=n2; ++k3)
					{
						nph3 = nph2*nphase(k3);
						HY3(k1,k2,k3) =   nph3*GY3(-k1,-k2,-k3);
						Hi3(k1,k2,k3) = - nph3*Gi3(-k1,-k2,-k3);
						H3(k1,k2,k3)  = dcmplx(HY3(k1,k2,k3),Hi3(k1,k2,k3)*pi);
						if ( nw>3 )
						{
							for (int k4=n1; k4<=n2; ++k4)
							{
								nph4 = nph3*nphase(k4);
								HY4(k1,k2,k3,k4) =   nph4*GY4(-k1,-k2,-k3,-k4);
								Hi4(k1,k2,k3,k4) = - nph4*Gi4(-k1,-k2,-k3,-k4);
								H4(k1,k2,k3,k4)  =
									 dcmplx(HY4(k1,k2,k3,k4),Hi4(k1,k2,k3,k4)*pi);
							}
						}
					}
				}
        	}
        }
	}
	if (n1==0) return;
	// correct the ill-defined entries
	HY2(-1,0) = - HY2(0,-1);
	Hi2(-1,0) = Hi1(0)*HY1(-1);
	H2(-1,0) = dcmplx(HY2(-1,0),Hi2(-1,0)*pi);
	if ( nw==2 ) return;
	HY3(-1,0,0) = HY1(-1)*HY2(0,0)+HY3(0,0,-1);
	Hi3(-1,0,0) = HY1(-1)*Hi2(0,0)-HY2(0,-1)*Hi1(0);
	H3(-1,0,0) = dcmplx(HY3(-1,0,0),Hi3(-1,0,0)*pi);
	if ( nw==3 ) return;
	HY4(-1,0,0,0) = -HY2(0,-1)*HY2(0,0)-HY4(0,0,0,-1);
	Hi4(-1,0,0,0) = HY1(-1)*Hi3(0,0,0)+HY3(0,0,-1)*Hi1(0);
	H4(-1,0,0,0) = dcmplx(HY4(-1,0,0,0),Hi4(-1,0,0,0)*pi);
	return;
}


/**
 *
 * @brief evaluates 1dhpl's in the (-1)-range  -(r2+1) < y <= -(r2-1)
 * evaluating first the H(..,-y) by calling eval1dhplat1(-y),
 * and then expressing H(..,y) in terms of H(..,-y)
 */
void hpl::details::eval1dhplatm1(double y, int nw,
		FA1dC11& H1,  FA2dC11& H2,  FA3dC11& H3,  FA4dC11& H4,
		FA1dD11& HY1, FA2dD11& HY2, FA3dD11& HY3, FA4dD11& HY4,
		FA1dD11& Hi1, FA2dD11& Hi2, FA3dD11& Hi3, FA4dD11& Hi4,
		int n1, int n2, int infilldim, FA1dI13 infill)
{
	// additional arrays required within this routine
	FA1dC11 G1;
	FA2dC11 G2;
	FA3dC11 G3;
	FA4dC11 G4;
	FA1dD11 GY1, Gi1;
	FA2dD11 GY2, Gi2;
	FA3dD11 GY3, Gi3;
	FA4dD11 GY4, Gi4;
	FA1dI13 istorfill;
	const int nphase_[3]={-1,1,-1};
	FA1dI11 nphase(nphase_);
	int nph1, nph2, nph3, nph4;
	// print*,' eval1dhplatm1: y = ',y
	if (infilldim==2)
	{
		for (int i=1; i<=2; ++i)
		{
			istorfill(i) = infill(i);
			infill(i) = -istorfill(i);
		}
	}
	// evaluate H(...,-y)
	setzero(nw,Gi1,Gi2,Gi3,Gi4,-n2,-n1);
	Gi1(0) = -1;
	eval1dhplat1(-y,nw,G1,G2,G3,G4,GY1,GY2,GY3,GY4,Gi1,Gi2,Gi3,Gi4,-n2,-n1,infilldim,infill);
	if (infilldim==2)
	{
		for (int i=1; i<=2; ++i)
		{
			infill(i) = istorfill(i);
		}
	}
	// fill the arrays H's
	for (int k1=n1; k1<=n2; ++k1)
	{
		nph1 = nphase(k1) ;
		HY1(k1) =   nph1*GY1(-k1);
		Hi1(k1) = - nph1*Gi1(-k1);
		H1(k1)  = dcmplx(HY1(k1),Hi1(k1)*pi);
		if ( nw>1 )
		{
			for (int k2=n1; k2<=n2; ++k2)
			{
				nph2 = nph1*nphase(k2);
				HY2(k1,k2) =   nph2*GY2(-k1,-k2);
				Hi2(k1,k2) = - nph2*Gi2(-k1,-k2);
				H2(k1,k2)  = dcmplx(HY2(k1,k2),Hi2(k1,k2)*pi);
				if ( nw>2 )
				{
					for (int k3=n1; k3<=n2; ++k3)
					{
						nph3 = nph2*nphase(k3);
						HY3(k1,k2,k3) =   nph3*GY3(-k1,-k2,-k3);
						Hi3(k1,k2,k3) = - nph3*Gi3(-k1,-k2,-k3);
						H3(k1,k2,k3)  = dcmplx(HY3(k1,k2,k3),Hi3(k1,k2,k3)*pi);
						if ( nw>3 )
						{
							for (int k4=n1; k4<=n2; ++k4)
							{
								nph4 = nph3*nphase(k4);
								HY4(k1,k2,k3,k4) =   nph4*GY4(-k1,-k2,-k3,-k4);
								Hi4(k1,k2,k3,k4) = - nph4*Gi4(-k1,-k2,-k3,-k4);
								H4(k1,k2,k3,k4)  = dcmplx(HY4(k1,k2,k3,k4),Hi4(k1,k2,k3,k4)*pi);
							}
						}
					}
				}
			}
        }
	}
	return;
}


/**
 *
 * @brief evaluates 1dhpl's in the (-1)-range y  <= -(r2+1)
 * evaluating first the H(..,-y) by calling eval1dhplatinf(-y),
 * and then expressing H(..,y) in terms of H(..,-y)
 */
void hpl::details::eval1dhplatminf(double y, int nw,
		FA1dC11& H1,  FA2dC11& H2,  FA3dC11& H3,  FA4dC11& H4,
		FA1dD11& HY1, FA2dD11& HY2, FA3dD11& HY3, FA4dD11& HY4,
		FA1dD11& Hi1, FA2dD11& Hi2, FA3dD11& Hi3, FA4dD11& Hi4,
		int n1, int n2, int infilldim, FA1dI13 infill)
{
	// additional arrays required within this routine
	FA1dC11 G1;
	FA2dC11 G2;
	FA3dC11 G3;
	FA4dC11 G4;
	FA1dD11 GY1, Gi1;
	FA2dD11 GY2, Gi2;
	FA3dD11 GY3, Gi3;
	FA4dD11 GY4, Gi4;
	FA1dI13 istorfill;
	const int nphase_[3]={-1,1,-1};
	FA1dI11 nphase(nphase_);
	int nph1, nph2, nph3, nph4;
	//      print*,' eval1dhplatm1: y = ',y
	if (infilldim==2)
	{
		for (int i=1; i<=2; ++i)
		{
			istorfill(i) = infill(i);
			infill(i) = -istorfill(i);
		}
	}
	// evaluate H(...,-y)
	setzero(nw,Gi1,Gi2,Gi3,Gi4,-n2,-n1);
	Gi1(0) = -1;
	eval1dhplatinf(-y,nw,G1,G2,G3,G4,GY1,GY2,GY3,GY4,Gi1,Gi2,Gi3,Gi4,-n2,-n1,infilldim,infill);
	if (infilldim==2)
	{
         for (int i=1; i<=2; ++i)
         {
            infill(i) = istorfill(i);
         }
	}
	// fill the arrays H's
	for (int k1=n1; k1<=n2; ++k1)
	{
		nph1 = nphase(k1);
		HY1(k1) =   nph1*GY1(-k1);
		Hi1(k1) = - nph1*Gi1(-k1);
		H1(k1)  = dcmplx(HY1(k1),Hi1(k1)*pi);
		if ( nw>1 )
		{
			for (int k2=n1; k2<=n2; ++k2)
            {
				nph2 = nph1*nphase(k2);
				HY2(k1,k2) =   nph2*GY2(-k1,-k2);
				Hi2(k1,k2) = - nph2*Gi2(-k1,-k2);
				H2(k1,k2)  = dcmplx(HY2(k1,k2),Hi2(k1,k2)*pi);
				if ( nw>2 )
				{
					for (int k3=n1; k3<=n2; ++k3)
					{
						nph3 = nph2*nphase(k3);
						HY3(k1,k2,k3) =   nph3*GY3(-k1,-k2,-k3);
						Hi3(k1,k2,k3) = - nph3*Gi3(-k1,-k2,-k3);
						H3(k1,k2,k3)  = dcmplx(HY3(k1,k2,k3),Hi3(k1,k2,k3)*pi);
						if ( nw>3 )
						{
							for (int k4=n1; k4<=n2; ++k4)
							{
								nph4 = nph3*nphase(k4);
								HY4(k1,k2,k3,k4) =   nph4*GY4(-k1,-k2,-k3,-k4);
								Hi4(k1,k2,k3,k4) = - nph4*Gi4(-k1,-k2,-k3,-k4);
								H4(k1,k2,k3,k4)  = dcmplx(HY4(k1,k2,k3,k4),Hi4(k1,k2,k3,k4)*pi);
							}
						}
					}
				}
            }
		}
	}
	return;
}


/**
 * @brief initializes with 0 the elements of the arrays
 */
void hpl::details::setzero(int nw, FA1dD11& Hi1, FA2dD11& Hi2, FA3dD11& Hi3, FA4dD11& Hi4,
		int n1, int n2)
{
	for (int k1=n1; k1<=n2; ++k1)
	{
		Hi1(k1) = 0.;
		if ( nw > 1 )
		{
			for (int k2=n1; k2<=n2; ++k2)
			{
				Hi2(k1,k2) = 0.;
				if ( nw > 2 )
					{
						for (int k3=n1; k3<=n2; ++k3)
						{
							Hi3(k1,k2,k3) = 0.;
							if ( nw > 3 )
								{
									for(int k4=n1; k4<=n2; ++k4)
									{
										Hi4(k1,k2,k3,k4) = 0.;
									}

								}
						}
					}
			}
		}
	}
	return;
}


/**
 *
 * @brief fills the reducible 1dhpl from the irreducible set
 */
void hpl::details::fillred1dhpl(int nw,
		FA1dC11& H1,  FA2dC11& H2,  FA3dC11& H3,  FA4dC11& H4,
		FA1dD11& HY1, FA2dD11& HY2, FA3dD11& HY3, FA4dD11& HY4,
		FA1dD11& Hi1, FA2dD11& Hi2, FA3dD11& Hi3, FA4dD11& Hi4,
		int n1, int n2, int infilldim, FA1dI13 infill)
{
	// combining real and immaginary into the complex value
	for (int k1=n1; k1<=n2; ++k1)
	{
		for (int k2=n1; k2<=n2; ++k2)
		{
			H2(k1,k2) = dcmplx(HY2(k1,k2),Hi2(k1,k2)*pi);
			if ( nw>2 )
			{
				for (int k3=n1; k3<=n2; ++k3)
				{
					H3(k1,k2,k3) = dcmplx(HY3(k1,k2,k3),Hi3(k1,k2,k3)*pi);
					if ( nw>3 )
					{
						for (int k4=n1; k4<=n2; ++k4)
						{
							H4(k1,k2,k3,k4) = dcmplx(HY4(k1,k2,k3,k4),Hi4(k1,k2,k3,k4)*pi);
						}
					}
				}
			}
		}
	}
	// evaluating the reduced HPL's
	// iflag = 0 to suppress auxiliary printings of FILLREDHPLx
	int iflag = 0;
	for (int ia = 1; ia <= infilldim; ++ia)
	{
		for (int ib = ia; ib <= infilldim; ++ib)
		{
			fillredhpl2(iflag,H1,H2,n1,n2,infill(ia),infill(ib));
			if ( nw>2 )
			{
				for (int ic = ib; ic<=infilldim; ++ic)
				{
					fillredhpl3(iflag,H1,H2,H3,n1,n2,infill(ia),infill(ib),infill(ic));
					if ( nw>3 )
					{
						for (int id = ic; id<=infilldim; ++id)
						{
							fillredhpl4(iflag,H1,H2,H3,H4,n1,n2,
									infill(ia),infill(ib),infill(ic),infill(id));
						}
					}
				}
			}
		}
	}
	// extractin real and immaginary parts from the complex value
	for (int k1=n1; k1<=n2; ++k1)
	{
		for (int k2=n1; k2<=n2; ++k2)
        {
			HY2(k1,k2) =  (H2(k1,k2)).real();
        	Hi2(k1,k2) = (H2(k1,k2)).imag()*pinv;
        	if ( nw>2 )
        	{
        		for (int k3=n1; k3<=n2; ++k3)
        		{
        			HY3(k1,k2,k3) = (H3(k1,k2,k3)).real();
        			Hi3(k1,k2,k3) = (H3(k1,k2,k3)).imag()*pinv;
					if ( nw>3 )
					{
						for (int k4=n1; k4<=n2; ++k4)
						{
							HY4(k1,k2,k3,k4) =  (H4(k1,k2,k3,k4)).real();
							Hi4(k1,k2,k3,k4) = (H4(k1,k2,k3,k4)).imag()*pinv;
						}
					}
        		}
        	}
        }
	}
	return;
}


/**
 *
 * @brief  must be called with ordered indices na <= nb
 */
void hpl::details::fillredhpl2(int iflag, FA1dC11& H1, FA2dC11& H2, int i1, int i2, int na, int nb)
{
	//      print*,' fillredhpl2, iflag =',iflag
	if ( na==nb )
	{
		H2(na,na) = 1./2.*std::pow(H1(na),2);
	}
	else
	{
		H2(nb,na) = H1(na)*H1(nb) - H2(na,nb);
        if ( iflag==1 )
        {
        	printer2(na,nb);
        }
	}
	return;
}


void hpl::details::fillredhpl3(int iflag, FA1dC11& H1, FA2dC11& H2, FA3dC11& H3,
		int i1, int i2, int ia, int ib, int ic)
{
	// must be called with "properly ordered" indices
	// note in particular the remapping of, say, (ia,ib,ic) into
	// (na,na,nb) of ReducerTest.out
	int na = ia;
	if ( (ia==ib) && (ib==ic) )
	{
		// case (na,na,na)
		H3(na,na,na) = 1./6.*std::pow(H1(na),3);
		// ic cannot be anymore equal to ia
	}
	else if ( ic==ia )
	{
		std::cerr << " fillredhpl3, error 1, called with arguments \n"
				"               " << ia << " " << ib << " " << ic << "\n";
        return;
	}
	else if ( ia==ib )
	{
		// case (na,na,nb)
		int nb = ic;
		if ( iflag==1 )
		{
			printer3(na,na,nb);
		}
		H3(na,nb,na) = H1(na)*H2(na,nb) - 2.*H3(na,na,nb);
		H3(nb,na,na) = 1./2.*H1(na)*H1(na)*H1(nb)
                    		  - H1(na)*H2(na,nb) + H3(na,na,nb);
	}
	// ib cannot be anymore equal to ia
	else if ( ib==ia )
	{
		std::cerr << " fillredhpl3, error 2, called with arguments \n"
				"               " << ia << " " << ib << " " << ic << "\n";
        return;
	}
	else if ( ib==ic )
	{
		// case (na,nb,nb)
		int nb = ib ;
		if ( iflag==1 )
		{
			printer3(na,nb,nb);
		}
		H3(nb,na,nb) = H1(nb)*H2(na,nb) - 2.*H3(na,nb,nb);
        H3(nb,nb,na) = 1./2.*H1(na)*H1(nb)*H1(nb)
                      - H1(nb)*H2(na,nb) + H3(na,nb,nb);
	}
	// no need to protect against ic==ib
	// when arriving here all indices are different
	else
	{
		// case (na,nb,nc)    all indices are different
		int nb = ib;
		int nc = ic;
        if ( iflag==1 )
        {
        	printer3(na,nb,nc);
        	printer3(na,nc,nb);
        }
        H3(nb,na,nc) = H1(nb)*H2(na,nc)
                      - H3(na,nb,nc) - H3(na,nc,nb);
        H3(nb,nc,na) = H1(na)*H2(nb,nc)
                      - H1(nb)*H2(na,nc) + H3(na,nc,nb);
        H3(nc,na,nb) = H1(nc)*H2(na,nb)
                      - H3(na,nb,nc) - H3(na,nc,nb);
        H3(nc,nb,na) = H1(na)*H1(nb)*H1(nc) - H1(na)*H2(nb,nc)
                      - H1(nc)*H2(na,nb) + H3(na,nb,nc);
	}
//*23456789012345678901234567890123456789012345678901234567890123456789012
	return;
}


void hpl::details::fillredhpl4(int iflag, FA1dC11& H1, FA2dC11& H2, FA3dC11& H3, FA4dC11& H4,
		int i1, int i2, int ia, int ib, int ic, int id)
{
	// must be called with "properly ordered" indices
	// note in particular the remapping of, say, (ia,ib,ic) into
	// (na,na,nb) of ReducerTest.out
	int na = ia;
	if ( (ia==ib) && (ib==ic) && (ic==id) )
	{
		// case (na,na,na,na)
        H4(na,na,na,na) = 1./24.*std::pow(H1(na),4);
        // id cannot be anymore equal to ia
	}
	else if ( id==ia )
	{
		std::cerr << " fillredhpl4, error 1, called with arguments \n"
				"               " << ia << " " << ib << " " << ic << " " << id << "\n";
        return;
	}
	else if ( (ia==ib) && (ib==ic) )
	{
		// case (na,na,na,nb)
        int nb = id;
        H4(na,na,nb,na) = H1(na)*H3(na,na,nb) - 3.*H4(na,na,na,nb);
        H4(na,nb,na,na) = 1./2.*H1(na)*H1(na)*H2(na,nb)
                         - 2.*H1(na)*H3(na,na,nb) + 3.*H4(na,na,na,nb);
        H4(nb,na,na,na) = 1./6.*H1(na)*H1(na)*H1(na)*H1(nb)
                         - 1./2.*H1(na)*H1(na)*H2(na,nb)
                         + H1(na)*H3(na,na,nb) - H4(na,na,na,nb);
        if ( iflag==1 )
        {
        	printer4(na,na,na,nb);
        }
        // ic cannot be anymore equal to ia
	}
	else if ( ic==ia )
	{
		std::cerr << " fillredhpl4, error 2, called with arguments \n"
        "               " << ia << " " << ib << " " << ic << " " << id << "\n";
		return;
	}
	else if ( (ia==ib) && (ic==id) )
	{
		// case (na,na,nb,nb)
        int nb = ic;
        H4(na,nb,na,nb) = 1./2.*H2(na,nb)*H2(na,nb)
                        - 2.*H4(na,na,nb,nb);
        H4(na,nb,nb,na) = H1(na)*H3(na,nb,nb)
                        - 1./2.*H2(na,nb)*H2(na,nb);
        H4(nb,na,na,nb) = H1(nb)*H3(na,na,nb)
                        - 1./2.*H2(na,nb)*H2(na,nb);
        H4(nb,na,nb,na) = H1(na)*H1(nb)*H2(na,nb)
                        - 2.*H1(na)*H3(na,nb,nb)
                        - 2.*H1(nb)*H3(na,na,nb)
                        + 1./2.*H2(na,nb)*H2(na,nb)
                        + 2.*H4(na,na,nb,nb);
        H4(nb,nb,na,na) = 1./4.*H1(na)*H1(na)*H1(nb)*H1(nb)
                        - H1(na)*H1(nb)*H2(na,nb)
                        + H1(na)*H3(na,nb,nb)
                        + H1(nb)*H3(na,na,nb) - H4(na,na,nb,nb);
        if ( iflag==1 )
        {
        	printer4(na,na,nb,nb);
        }
	}
	else if ( ia==ib )
	{
		// case (na,na,nb,nc)
		int nb = ic;
		int nc = id;
		H4(na,nb,nc,na) = H1(na)*H3(na,nb,nc) - 2.*H4(na,na,nb,nc)
						- H4(na,nb,na,nc);
		H4(na,nc,na,nb) = H2(na,nb)*H2(na,nc) - 2.*H4(na,na,nb,nc)
						- 2.*H4(na,na,nc,nb) - H4(na,nb,na,nc);
		H4(na,nc,nb,na) = H1(na)*H3(na,nc,nb) - H2(na,nb)*H2(na,nc)
						+ 2.*H4(na,na,nb,nc) + H4(na,nb,na,nc);
		H4(nb,na,na,nc) = H1(nb)*H3(na,na,nc) - H4(na,na,nb,nc)
						- H4(na,na,nc,nb) - H4(na,nb,na,nc);
		H4(nb,na,nc,na) = H1(na)*H1(nb)*H2(na,nc)
						- H1(na)*H3(na,nb,nc) - H1(na)*H3(na,nc,nb)
						- 2.*H1(nb)*H3(na,na,nc) + 2.*H4(na,na,nb,nc)
						+ 2.*H4(na,na,nc,nb) + H4(na,nb,na,nc);
		H4(nb,nc,na,na) = 1./2.*H1(na)*H1(na)*H2(nb,nc)
						- H1(na)*H1(nb)*H2(na,nc)
						+ H1(na)*H3(na,nc,nb) + H1(nb)*H3(na,na,nc)
						- H4(na,na,nc,nb);
		H4(nc,na,na,nb) = H1(nc)*H3(na,na,nb) - H2(na,nb)*H2(na,nc)
						 + H4(na,na,nb,nc) + H4(na,na,nc,nb)
						 + H4(na,nb,na,nc);
		H4(nc,na,nb,na) = H1(na)*H1(nc)*H2(na,nb)
						- H1(na)*H3(na,nb,nc) - H1(na)*H3(na,nc,nb)
						- 2.*H1(nc)*H3(na,na,nb) + H2(na,nb)*H2(na,nc)
						- H4(na,nb,na,nc);
		H4(nc,nb,na,na) = 1./2.*H1(na)*H1(na)*H1(nb)*H1(nc)
						- 1./2.*H1(na)*H1(na)*H2(nb,nc)
						- H1(na)*H1(nc)*H2(na,nb)
						+ H1(na)*H3(na,nb,nc) + H1(nc)*H3(na,na,nb)
						- H4(na,na,nb,nc);
		if ( iflag==1 )
		{
			printer4(na,na,nb,nc);
			printer4(na,na,nc,nb);
			printer4(na,nb,na,nc);
		}
	}
	// ib cannot be anymore equal to ia
	else if ( ib==ia )
	{
		std::cerr << " fillredhpl4, error 3, called with arguments \n"
				"               " << ia << " " << ib << " " << ic << " " << id << "\n";
		return;
	}
	else if ( (ib==ic) && (ic==id) )
	{
		// case (na,nb,nb,nb)
		int nb = ib;
		H4(nb,na,nb,nb) = H1(nb)*H3(na,nb,nb) - 3.*H4(na,nb,nb,nb);
		H4(nb,nb,na,nb) = 1./2.*H1(nb)*H1(nb)*H2(na,nb)
						- 2.*H1(nb)*H3(na,nb,nb) + 3.*H4(na,nb,nb,nb);
		H4(nb,nb,nb,na) = 1./6.*H1(na)*H1(nb)*H1(nb)*H1(nb)
						- 1./2.*H1(nb)*H1(nb)*H2(na,nb)
						+ H1(nb)*H3(na,nb,nb) - H4(na,nb,nb,nb);
		if ( iflag==1 )
		{
			printer4(na,nb,nb,nb);
		}
	}
	// id cannot be anymore equal to ib
	else if ( id==ib )
	{
		std::cerr << " fillredhpl4, error 4, called with arguments \n"
				"               " << ia << " " << ib << " " << ic << " " << id << "\n";
		return;
	}
	else if ( ib==ic )
	{
		// case (na,nb,nb,nc)
		int nb = ib;
		int nc = id;
		H4(nb,na,nb,nc) = H1(nb)*H3(na,nb,nc)
						- 2.*H4(na,nb,nb,nc) - H4(na,nb,nc,nb);
		H4(nb,na,nc,nb) = H1(nb)*H3(na,nc,nb) - H4(na,nb,nc,nb)
						- 2.*H4(na,nc,nb,nb);
		H4(nb,nb,na,nc) = 1./2.*H1(nb)*H1(nb)*H2(na,nc)
						- H1(nb)*H3(na,nb,nc) - H1(nb)*H3(na,nc,nb)
						+ H4(na,nb,nb,nc) + H4(na,nb,nc,nb)
						+ H4(na,nc,nb,nb);
		H4(nb,nb,nc,na) = H1(na)*H3(nb,nb,nc)
						- 1./2.*H1(nb)*H1(nb)*H2(na,nc)
						+ H1(nb)*H3(na,nc,nb) - H4(na,nc,nb,nb);
		H4(nb,nc,na,nb) = - H1(nb)*H3(na,nb,nc) - H1(nb)*H3(na,nc,nb)
						+ H2(na,nb)*H2(nb,nc) + H4(na,nb,nc,nb)
						+ 2.*H4(na,nc,nb,nb);
		H4(nb,nc,nb,na) = H1(na)*H1(nb)*H2(nb,nc)
						- 2.*H1(na)*H3(nb,nb,nc)
						+ H1(nb)*H3(na,nb,nc)
						- H2(na,nb)*H2(nb,nc) - H4(na,nb,nc,nb);
		H4(nc,na,nb,nb) = H1(nc)*H3(na,nb,nb) - H4(na,nb,nb,nc)
						- H4(na,nb,nc,nb) - H4(na,nc,nb,nb);
		H4(nc,nb,na,nb) = H1(nb)*H1(nc)*H2(na,nb)
						- 2.*H1(nc)*H3(na,nb,nb)
						- H2(na,nb)*H2(nb,nc) + 2.*H4(na,nb,nb,nc)
						+ H4(na,nb,nc,nb);
		H4(nc,nb,nb,na) = 1./2.*H1(na)*H1(nb)*H1(nb)*H1(nc)
						- H1(na)*H1(nb)*H2(nb,nc)
						+ H1(na)*H3(nb,nb,nc)
						- H1(nb)*H1(nc)*H2(na,nb)
						+ H1(nc)*H3(na,nb,nb) + H2(na,nb)*H2(nb,nc)
						- H4(na,nb,nb,nc);
		if ( iflag==1 )
		{
			printer4(na,nb,nb,nc);
			printer4(na,nb,nc,nb);
			printer4(na,nc,nb,nb);
		}
	}
    // ic cannot be anymore equal to ib
	else if ( ic==ib )
	{
        std::cerr << " fillredhpl4, error 5, called with arguments \n"
        		"               " << ia << " " << ib << " " << ic << " " << id << "\n";
        return;
	}
	else if ( ic==id )
	{
		// case (na,nb,nc,nc)
		int nb = ib;
		int nc = ic;
        H4(nb,na,nc,nc) = H1(nb)*H3(na,nc,nc) - H4(na,nb,nc,nc)
                         - H4(na,nc,nb,nc) - H4(na,nc,nc,nb);
        H4(nb,nc,na,nc) = - 2.*H1(nb)*H3(na,nc,nc) + H2(na,nc)*H2(nb,nc)
                         + H4(na,nc,nb,nc) + 2.*H4(na,nc,nc,nb);
        H4(nb,nc,nc,na) = H1(na)*H3(nb,nc,nc) + H1(nb)*H3(na,nc,nc)
                         - H2(na,nc)*H2(nb,nc) - H4(na,nc,nc,nb);
        H4(nc,na,nb,nc) = H1(nc)*H3(na,nb,nc) - 2.*H4(na,nb,nc,nc)
                         - H4(na,nc,nb,nc);
        H4(nc,na,nc,nb) = H1(nc)*H3(na,nc,nb) - H4(na,nc,nb,nc)
                         - 2.*H4(na,nc,nc,nb);
        H4(nc,nb,na,nc) = H1(nb)*H1(nc)*H2(na,nc)
                         - H1(nc)*H3(na,nb,nc) - H1(nc)*H3(na,nc,nb)
                         - H2(na,nc)*H2(nb,nc) + 2.*H4(na,nb,nc,nc)
                         + H4(na,nc,nb,nc);
        H4(nc,nb,nc,na) = H1(na)*H1(nc)*H2(nb,nc)
                         - 2.*H1(na)*H3(nb,nc,nc)
                         - H1(nb)*H1(nc)*H2(na,nc)
                         + H1(nc)*H3(na,nc,nb) + H2(na,nc)*H2(nb,nc)
                         - H4(na,nc,nb,nc);
        H4(nc,nc,na,nb) = 1./2.*H1(nc)*H1(nc)*H2(na,nb)
                         - H1(nc)*H3(na,nb,nc) - H1(nc)*H3(na,nc,nb)
                         + H4(na,nb,nc,nc) + H4(na,nc,nb,nc)
                         + H4(na,nc,nc,nb);
        H4(nc,nc,nb,na) = 1./2.*H1(na)*H1(nb)*H1(nc)*H1(nc)
                         - H1(na)*H1(nc)*H2(nb,nc)
                         + H1(na)*H3(nb,nc,nc)
                         - 1./2.*H1(nc)*H1(nc)*H2(na,nb)
                         + H1(nc)*H3(na,nb,nc) - H4(na,nb,nc,nc);
        if ( iflag==1 ){
        	printer4(na,nb,nc,nc);
			printer4(na,nc,nb,nc);
			printer4(na,nc,nc,nb);
        }
	}
	// no need to protect against id==ic
	// when arriving here all indices are different
	else
	{
		// case (na,nb,nc,nd) all indices are different
        int nb = ib;
        int nc = ic;
        int nd = id;
        H4(nb,na,nc,nd) = H1(nb)*H3(na,nc,nd) - H4(na,nb,nc,nd)
                         - H4(na,nc,nb,nd) - H4(na,nc,nd,nb);
        H4(nb,na,nd,nc) = H1(nb)*H3(na,nd,nc) - H4(na,nb,nd,nc)
                         - H4(na,nd,nb,nc) - H4(na,nd,nc,nb);
        H4(nb,nc,na,nd) = - H1(nb)*H3(na,nc,nd) - H1(nb)*H3(na,nd,nc) 
                         + H2(na,nd)*H2(nb,nc) + H4(na,nc,nb,nd)
                         + H4(na,nc,nd,nb) + H4(na,nd,nc,nb);
        H4(nb,nc,nd,na) = H1(na)*H3(nb,nc,nd) + H1(nb)*H3(na,nd,nc)
                         - H2(na,nd)*H2(nb,nc) - H4(na,nd,nc,nb);
        H4(nb,nd,na,nc) = - H1(nb)*H3(na,nc,nd) - H1(nb)*H3(na,nd,nc) 
                         + H2(na,nc)*H2(nb,nd) + H4(na,nc,nd,nb)
                         + H4(na,nd,nb,nc) + H4(na,nd,nc,nb);
        H4(nb,nd,nc,na) = H1(na)*H3(nb,nd,nc) + H1(nb)*H3(na,nc,nd)
                         - H2(na,nc)*H2(nb,nd) - H4(na,nc,nd,nb);
        H4(nc,na,nb,nd) = H1(nc)*H3(na,nb,nd) - H4(na,nb,nc,nd)
                         - H4(na,nb,nd,nc) - H4(na,nc,nb,nd);
        H4(nc,na,nd,nb) = H1(nc)*H3(na,nd,nb) - H4(na,nc,nd,nb)
                         - H4(na,nd,nb,nc) - H4(na,nd,nc,nb);
        H4(nc,nb,na,nd) = H1(nb)*H1(nc)*H2(na,nd)
                         - H1(nc)*H3(na,nb,nd) - H1(nc)*H3(na,nd,nb)
                         - H2(na,nd)*H2(nb,nc) + H4(na,nb,nc,nd)
                         + H4(na,nb,nd,nc) + H4(na,nd,nb,nc);
        H4(nc,nb,nd,na) = H1(na)*H1(nc)*H2(nb,nd)
                         - H1(na)*H3(nb,nc,nd) - H1(na)*H3(nb,nd,nc)
                         - H1(nb)*H1(nc)*H2(na,nd)
                         + H1(nc)*H3(na,nd,nb) + H2(na,nd)*H2(nb,nc)
                         - H4(na,nd,nb,nc);
        H4(nc,nd,na,nb) = - H1(nc)*H3(na,nb,nd) - H1(nc)*H3(na,nd,nb) 
                         + H2(na,nb)*H2(nc,nd) + H4(na,nb,nd,nc)
                         + H4(na,nd,nb,nc) + H4(na,nd,nc,nb);
        H4(nc,nd,nb,na) = H1(na)*H1(nb)*H2(nc,nd)
                         - H1(na)*H1(nc)*H2(nb,nd)
                         + H1(na)*H3(nb,nd,nc) + H1(nc)*H3(na,nb,nd)
                         - H2(na,nb)*H2(nc,nd) - H4(na,nb,nd,nc);
        H4(nd,na,nb,nc) = H1(nd)*H3(na,nb,nc) - H4(na,nb,nc,nd)
                         - H4(na,nb,nd,nc) - H4(na,nd,nb,nc);
        H4(nd,na,nc,nb) = H1(nd)*H3(na,nc,nb) - H4(na,nc,nb,nd)
                         - H4(na,nc,nd,nb) - H4(na,nd,nc,nb);
        H4(nd,nb,na,nc) = H1(nb)*H1(nd)*H2(na,nc)
                         - H1(nd)*H3(na,nb,nc) - H1(nd)*H3(na,nc,nb)
                         - H2(na,nc)*H2(nb,nd) + H4(na,nb,nc,nd)
                         + H4(na,nb,nd,nc) + H4(na,nc,nb,nd);
        H4(nd,nb,nc,na) = H1(na)*H1(nd)*H2(nb,nc)
                         - H1(na)*H3(nb,nc,nd) - H1(na)*H3(nb,nd,nc)
                         - H1(nb)*H1(nd)*H2(na,nc)
                         + H1(nd)*H3(na,nc,nb) + H2(na,nc)*H2(nb,nd)
                         - H4(na,nc,nb,nd);
        H4(nd,nc,na,nb) = H1(nc)*H1(nd)*H2(na,nb)
                         - H1(nd)*H3(na,nb,nc) - H1(nd)*H3(na,nc,nb)
                         - H2(na,nb)*H2(nc,nd) + H4(na,nb,nc,nd)
                         + H4(na,nc,nb,nd) + H4(na,nc,nd,nb);
        H4(nd,nc,nb,na) = H1(na)*H1(nb)*H1(nc)*H1(nd)
                         - H1(na)*H1(nb)*H2(nc,nd)
                         - H1(na)*H1(nd)*H2(nb,nc)
                         + H1(na)*H3(nb,nc,nd)
                         - H1(nc)*H1(nd)*H2(na,nb)
                         + H1(nd)*H3(na,nb,nc)
                         + H2(na,nb)*H2(nc,nd) - H4(na,nb,nc,nd);
        if ( iflag==1 )
        {
			printer4(na,nb,nc,nd);
			printer4(na,nb,nd,nc);
			printer4(na,nc,nb,nd);
			printer4(na,nc,nb,nd);
			printer4(na,nd,nb,nc);
			printer4(na,nd,nc,nb);
        }
	}
//*23456789012345678901234567890123456789012345678901234567890123456789012
	return;
}


void hpl::details::printer2(int na, int nb)
{

//      write(11,'(''g [H('',)')
//      call subprint(11,na)
//      write(11,'('','',)')
//      call subprint(11,nb)
//      write(11,'('',y)] = H('',)')
//      call subprint(11,na)
//      write(11,'('','',)')
//      call subprint(11,nb)
//      write(11,'('',y) ; '')')
//
//      write(12,'(''id H('',)')
//      call subprint(12,na)
//      write(12,'('','',)')
//      call subprint(12,nb)
//      write(12,'('',y) = H[('',)')
//      call subprint(12,na)
//      write(12,'('','',)')
//      call subprint(12,nb)
//      write(12,'('',y)] ; '')')

      return;
};


void hpl::details::printer3(int na, int nb, int nc)
{
//      write(11,'(''g [H('',)')
//      call subprint(11,na)
//      write(11,'('','',)')
//      call subprint(11,nb)
//      write(11,'('','',)')
//      call subprint(11,nc)
//      write(11,'('',y)] = H('',)')
//      call subprint(11,na)
//      write(11,'('','',)')
//      call subprint(11,nb)
//      write(11,'('','',)')
//      call subprint(11,nc)
//      write(11,'('',y) ; '')')
//
//      write(12,'(''id H('',)')
//      call subprint(12,na)
//      write(12,'('','',)')
//      call subprint(12,nb)
//      write(12,'('','',)')
//      call subprint(12,nc)
//      write(12,'('',y) = H[('',)')
//      call subprint(12,na)
//      write(12,'('','',)')
//      call subprint(12,nb)
//      write(12,'('',y)] ; '')')

      return;
}


void hpl::details::printer4(int na, int nb, int nc, int nd)
{
//      write(11,'(''g [H('',)')
//      call subprint(11,na)
//      write(11,'('','',)')
//      call subprint(11,nb)
//      write(11,'('','',)')
//      call subprint(11,nc)
//      write(11,'('','',)')
//      call subprint(11,nd)
//      write(11,'('',y)] = H('',)')
//      call subprint(11,na)
//      write(11,'('','',)')
//      call subprint(11,nb)
//      write(11,'('','',)')
//      call subprint(11,nc)
//      write(11,'('','',)')
//      call subprint(11,nd)
//      write(11,'('',y) ; '')')
//
//      write(12,'(''id H('',)')
//      call subprint(12,na)
//      write(12,'('','',)')
//      call subprint(12,nb)
//      write(12,'('','',)')
//      call subprint(12,nc)
//      write(12,'('','',)')
//      call subprint(12,nd)
//      write(12,'('',y) = H[('',)')
//      call subprint(12,na)
//      write(12,'('','',)')
//      call subprint(12,nb)
//      write(12,'('','',)')
//      call subprint(12,nc)
//      write(12,'('','',)')
//      call subprint(12,nd)
//      write(12,'('',y)] ; '')')

      return;
}


void hpl::details::subprint(int n, int na)
{
//      if ( na<0 ){
//        write (n,102) na
//      else
//        write (n,101) na
//      endif
//      return
//  101 format(i1,)
//  102 format(i2,)
}


/************************************************************************
** the following routines contain the set of routines evaluating
** irreducible 1dhpl's for various values of the arguments 
************************************************************************/
void hpl::details::fillh1(double y, FA1dC11& H1, FA1dD11& HY1, FA1dD11& Hi1, int n1, int n2)
{
	// fillh1 evaluates the 1dhpl's of weight 1
	if ( n1==-1)
	{
		if ( y>=-1. )
		{
			HY1(-1) = log(1.+y);
			Hi1(-1) = 0.;
		}
		else if ( y<-1. )
		{
			HY1(-1) = log(-1.-y);
			Hi1(-1) = 1.;
		}
		H1(-1) = dcmplx(HY1(-1),pi*Hi1(-1));
	}
	if ( y>=0. )
	{
		HY1(0) = log(y);
		//        Hi1(0) = 0.
	}
	else if ( y<0. ){
        HY1(0) = log(-y);
        Hi1(0) = 1.;
	}
	H1(0) = dcmplx(HY1(0),pi*Hi1(0));
	if ( n2==1 )
	{
		if ( y>=1. )
		{
			HY1(1) = - log(-1.+y);
			Hi1(1) = 1.;
        } else if ( y<1. ){
			HY1(1) = - log(1.-y);
			Hi1(1) = 0.;
        }
		H1(1) = dcmplx(HY1(1),pi*Hi1(1));
	}
	return;
}


/**
 *
 * @brief evaluate the HPL from their power series expansions
 * fillirr1dhplat0 is called by eval1dhplat0;
 * it is guaranteed that nw is in the range 1:4, and that (n1,n2)
 * take one of the pairs of values (0,1), (-1,0) or (-1,1)
 * for y < 0 DOES NOT evaluates the immaginary part of H(0,y) = log(y)
 */
void hpl::details::fillirr1dhplat0(double y, int nw,
		FA1dD11& HY1, FA2dD11& HY2, FA3dD11& HY3, FA4dD11& HY4,
		int n1, int n2)
{
	// evaluating the required 1dHPL of weight 1
	double u,v, ep, e2;
	double tu01, tu02, tu03, tu04, tu05, tu06, tu07, tu08, tu09, tu10, tu11, tu12;
	double tv01, tv02, tv03, tv04, tv05, tv06, tv07, tv08, tv09, tv10, tv11, tv12;
	if ( n1==-1)
	{
	// 1+y = (1+ep)/(1-ep), ep = y/(2+y)
	// log(1+y) = log((1+y)/(1-y)) = 2*ep*(1+ep^2/3+ep^4/5+.....)
	// at y= -(r2-1) = - 0.4142135624, ep = - 0.26120387496
	// ep2 = 0.068227464296, ep2^13 = 6.9 x 10^(-16)
		ep = y/(2.+y);
		e2 = ep*ep;
		//         v = log(1.+y)
		v = 2*ep*(1+e2*(1./3. + e2*(1./5. + e2*(1./7. + e2*(1./9.
        		+ e2*(1./11. + e2*(1./13. + e2*(1./15. + e2*(1./17.
        		+ e2*(1./19. + e2*(1./21. + e2*(1./25. + e2*(1./27.
        		)))))))))))));
         HY1(-1) = v;
	}
	if (y>=0.)
	{
		HY1(0) = log(y);
	}
	else
	{
		HY1(0) = log(-y);
	}

	// the immaginary part is evaluated in the calling routine eval1dhplat0
	//       Hi1(0) = 1.
	if ( n2==1)
	{
		// 1-y = (1-ep)/(1+ep), ep = y/(2-y)
		ep = y/(2.-y);
		e2 = ep*ep;
		// u = - log(1.-y)
		u = 2*ep*(1. + e2*(1./3. + e2*(1./5. + e2*(1./7. + e2*(1./9.
                   + e2*(1./11. + e2*(1./13. + e2*(1./15. + e2*(1./17.
                   + e2*(1./19. + e2*(1./21. + e2*(1./25. + e2*(1./27.
                   )))))))))))));
         HY1(1) = u;
	}
	if ( nw==1 ) return;
	// from now on nw > 1
	// evaluating the Cebyshev polynomials for the expansions
	ep = y;
	if ( n2==1)
	{
		tu01 = 20./11.*u;
        tu02 = 2.*tu01*tu01 - 1.;
        tu03 = 2.*tu01*tu02 - tu01;
        tu04 = 2.*tu01*tu03 - tu02;
        tu05 = 2.*tu01*tu04 - tu03;
        tu06 = 2.*tu01*tu05 - tu04;
        tu07 = 2.*tu01*tu06 - tu05;
        tu08 = 2.*tu01*tu07 - tu06;
        tu09 = 2.*tu01*tu08 - tu07;
        tu10 = 2.*tu01*tu09 - tu08;
        tu11 = 2.*tu01*tu10 - tu09;
        tu12 = 2.*tu01*tu11 - tu10;
	}
	if ( n1==-1 ){
		tv01 = 20./11.*v;
		tv02 = 2.*tv01*tv01 - 1.;
		tv03 = 2.*tv01*tv02 - tv01;
		tv04 = 2.*tv01*tv03 - tv02;
		tv05 = 2.*tv01*tv04 - tv03;
		tv06 = 2.*tv01*tv05 - tv04;
		tv07 = 2.*tv01*tv06 - tv05;
		tv08 = 2.*tv01*tv07 - tv06;
		tv09 = 2.*tv01*tv08 - tv07;
		tv10 = 2.*tv01*tv09 - tv08;
		tv11 = 2.*tv01*tv10 - tv09;
		tv12 = 2.*tv01*tv11 - tv10;
	}
	// evaluating the expansions
	// (n1,n2) = (0,1) or (-1,1)
	if ( ( (n1==0) && (n2==1) ) || ( (n1==-1) && (n2==1) ) )
	{
		HY2(0,1) =
		   - 3.781250000000000e-02
		   + 5.534574473824441e-01*tu01
		   - 3.781250000000000e-02*tu02
		   + 1.151036617760703e-03*tu03
		   - 8.659502433858922e-07*tu05
		   + 1.109042494804544e-09*tu07
		   - 1.624415058184216e-12*tu09
		   + 2.528376460336939e-15*tu11;
		// it would be wrong to write
		// if ( nw==2 ) return
		// because the (n1==-1) && (n2==1) case is not yet complete
		if ( nw>2 )
		{
			HY3(0,0,1) =
			   - 5.701592410758114e-02
			   + 5.598247957892565e-01*tu01
			   - 5.711486614505007e-02*tu02
			   + 3.275603992203700e-03*tu03
			   - 9.887255877938583e-05*tu04
			   + 4.021153684652295e-07*tu05
			   + 6.939288687864526e-08*tu06
			   - 7.995347631322020e-10*tu07
			   - 8.567978673919505e-11*tu08
			   + 1.526387027481200e-12*tu09
			   + 1.226899454816980e-13*tu10
			   - 2.848614761014972e-15*tu11
			   - 1.880542777479446e-16*tu12;
			HY3(0,1,1) =
			   + 3.816894981500984e-02
			   - 1.039843750000000e-02*tu01
			   + 3.828760080995617e-02*tu02
			   - 3.466145833333333e-03*tu03
			   + 1.185518160084905e-04*tu04
			   - 9.904555648775859e-08*tu06
			   + 1.331803984518588e-10*tu08
			   - 2.006389465106708e-13*tu10
			   + 3.180731062055677e-16*tu12;
		}
		if ( nw>3 )
		{
			HY4(0,0,0,1) =
				- 6.685228257646101e-02
				+ 5.645990701998083e-01*tu01
				- 6.707912936340146e-02*tu02
				+ 4.876429488624746e-03*tu03
				- 2.268732672568699e-04*tu04
				+ 6.038494106229146e-06*tu05
				- 2.642577015932576e-08*tu06
				- 3.679843316593900e-09*tu07
				+ 5.444046563879984e-11*tu08
				+ 4.063821221202881e-12*tu09
				- 1.055985864474070e-13*tu10
				- 5.190408125225683e-15*tu11
				+ 1.985464489219049e-16*tu12;
			HY4(0,0,1,1) =
				+ 1.953236111099851e-02
				- 8.741612828671381e-03*tu01
				+ 1.974116110893196e-02*tu02
				- 2.926558492394004e-03*tu03
				+ 2.088576190269387e-04*tu04
				- 7.604351107741397e-06*tu05
				+ 5.751031394942524e-08*tu06
				+ 5.832253077603139e-09*tu07
				- 1.105713721511985e-10*tu08
				- 7.453416210082473e-12*tu09
				+ 2.077758906032370e-13*tu10
				+ 1.085601519719514e-14*tu11
				- 3.848312092795918e-16*tu12;
			HY4(0,1,1,1) =
				- 7.148925781250000e-04
				+ 7.019393481825299e-03*tu01
				- 9.531901041666666e-04*tu02
				+ 2.354287493676137e-03*tu03
				- 2.382975260416666e-04*tu04
				+ 8.682904829408987e-06*tu05
				- 7.768198634676578e-09*tu07
				+ 1.083130072188330e-11*tu09
				- 1.668810490326842e-14*tu11;
		} // nw > 3 endif
	} // (n1,n2) = (0,1) or (-1,1) endif

	//************
	// (n1,n2) = (-1,0) or (-1,1)
	if (  ( (n1==-1) && (n2==0) ) || ( (n1==-1) && (n2==1) ) )
	{
		HY2(0,-1) =
			+ 3.781250000000000e-02
			+ 5.534574473824441e-01*tv01
			+ 3.781250000000000e-02*tv02
			+ 1.151036617760703e-03*tv03
			- 8.659502433858922e-07*tv05
			+ 1.109042494804544e-09*tv07
			- 1.624415058184216e-12*tv09
			+ 2.528376460336939e-15*tv11;
		if ( nw>2 )
		{
			HY3(0,0,-1) =
				+ 5.701592410758114e-02
				+ 5.598247957892565e-01*tv01
				+ 5.711486614505007e-02*tv02
				+ 3.275603992203700e-03*tv03
				+ 9.887255877938583e-05*tv04
				+ 4.021153684652295e-07*tv05
				- 6.939288687864526e-08*tv06
				- 7.995347631322020e-10*tv07
				+ 8.567978673919505e-11*tv08
				+ 1.526387027481200e-12*tv09
				- 1.226899454816980e-13*tv10
				- 2.848614761014972e-15*tv11
				+ 1.880542777479446e-16*tv12;
			HY3(0,-1,-1) =
				+ 3.816894981500984e-02
				+ 1.039843750000000e-02*tv01
				+ 3.828760080995617e-02*tv02
				+ 3.466145833333333e-03*tv03
				+ 1.185518160084905e-04*tv04
				- 9.904555648775859e-08*tv06
				+ 1.331803984518588e-10*tv08
				- 2.006389465106708e-13*tv10
				+ 3.180731062055677e-16*tv12;
		}
		if ( nw>3 )
		{
			HY4(0,0,0,-1) =
				+ 6.685228257646101e-02
				+ 5.645990701998083e-01*tv01
				+ 6.707912936340146e-02*tv02
				+ 4.876429488624746e-03*tv03
				+ 2.268732672568699e-04*tv04
				+ 6.038494106229146e-06*tv05
				+ 2.642577015932576e-08*tv06
				- 3.679843316593900e-09*tv07
				- 5.444046563879984e-11*tv08
				+ 4.063821221202881e-12*tv09
				+ 1.055985864474070e-13*tv10
				- 5.190408125225683e-15*tv11
				- 1.985464489219049e-16*tv12;
			HY4(0,0,-1,-1) =
				+ 1.953236111099851e-02
				+ 8.741612828671381e-03*tv01
				+ 1.974116110893196e-02*tv02
				+ 2.926558492394004e-03*tv03
				+ 2.088576190269387e-04*tv04
				+ 7.604351107741397e-06*tv05
				+ 5.751031394942524e-08*tv06
				- 5.832253077603139e-09*tv07
				- 1.105713721511985e-10*tv08
				+ 7.453416210082473e-12*tv09
				+ 2.077758906032370e-13*tv10
				- 1.085601519719514e-14*tv11
				- 3.848312092795918e-16*tv12;
			HY4(0,-1,-1,-1) =
				+ 7.148925781250000e-04
				+ 7.019393481825299e-03*tv01
				+ 9.531901041666666e-04*tv02
				+ 2.354287493676137e-03*tv03
				+ 2.382975260416666e-04*tv04
				+ 8.682904829408987e-06*tv05
				- 7.768198634676578e-09*tv07
				+ 1.083130072188330e-11*tv09
				- 1.668810490326842e-14*tv11;
		} // nw > 3 endif
	} // (n1,n2) = (-1,0) or (-1,1) endif
	// (n1,n2) = (-1,1) -- completion
	if ( (n1==-1) && (n2==1) )
	{
		HY2(-1,1) =
			- 2.924454241163343e-02
			+ 3.845279287117326e-01*tu01
			- 2.925485694830038e-02*tu02
			+ 1.097780471057338e-03*tu03
			- 1.029703135442673e-05*tu04
			- 7.265175511511970e-07*tu05
			+ 1.747461299829753e-08*tu06
			+ 7.707353556013722e-10*tu07
			- 3.064611747990741e-11*tu08
			- 8.531228176305706e-13*tu09
			+ 5.331187822989144e-14*tu10
			+ 8.500141365188675e-16*tu11
			- 6.931471805599453e-01*HY1(-1);
		if ( nw>2 )
		{
			HY3(0,-1,1) =
				- 4.107537580582269e-02
				+ 3.887609555197323e-01*tu01
				- 4.116162793629221e-02*tu02
				+ 2.511526558054413e-03*tu03
				- 8.620496933228561e-05*tu04
				+ 9.128023201466990e-07*tu05
				+ 4.711634663963971e-08*tu06
				- 1.347359673414334e-09*tu07
				- 4.474345520888852e-11*tu08
				+ 2.138249646727980e-12*tu09
				+ 4.709915818801180e-14*tu10
				- 3.454431385666621e-15*tu11
				- 6.931471805599453e-01*HY2(0,-1);
			HY3(0,1,-1) =
				- 4.107537580582269e-02
				- 3.887609555197323e-01*tv01
				- 4.116162793629221e-02*tv02
				- 2.511526558054413e-03*tv03
				- 8.620496933228561e-05*tv04
				- 9.128023201466990e-07*tv05
				+ 4.711634663963971e-08*tv06
				+ 1.347359673414334e-09*tv07
				- 4.474345520888852e-11*tv08
				- 2.138249646727980e-12*tv09
				+ 4.709915818801180e-14*tv10
				+ 3.454431385666621e-15*tv11
				+ 6.931471805599453e-01*HY2(0,1);
			HY3(-1,-1,1) =
				- 3.590863871372201e-02
				+ 3.272029419300922e-01*tu01
				- 3.599657175069328e-02*tu02
				+ 2.325685169395631e-03*tu03
				- 8.788997314012583e-05*tu04
				+ 1.277831858501559e-06*tu05
				+ 4.303730428865162e-08*tu06
				- 1.992295216809703e-09*tu07
				- 2.652932076676834e-11*tu08
				+ 3.159865930142703e-12*tu09
				- 2.395589527593406e-15*tu10
				- 4.870947810519399e-15*tu11
				- 5.822405264650125e-01*HY1(-1)
				- 3.465735902799726e-01*HY1(-1)*HY1(-1);
			HY3(-1,1,1) =
				+ 3.668493142404161e-02
				- 1.413123104773291e-01*tu01
				+ 3.680167312678666e-02*tu02
				- 3.064044728536094e-03*tu03
				+ 1.166524199994130e-04*tu04
				- 8.779983417383380e-07*tu05
				- 8.917940330502000e-08*tu06
				+ 1.787575622706040e-09*tu07
				+ 1.032182649980912e-10*tu08
				- 3.441821872732193e-12*tu09
				- 1.239218730863368e-13*tu10
				+ 6.355731482672869e-15*tu11
				+ 1.386175839607904e-16*tu12
				+ 2.402265069591007e-01*HY1(-1);
		}
		if ( nw>3 )
		{
			HY4(0,0,-1,1) =
				- 4.713463351559199e-02
				+ 3.918037828258655e-01*tu01
				- 4.730698763577787e-02*tu02
				+ 3.532784273601097e-03*tu03
				- 1.724036773635937e-04*tu04
				+ 5.100573466380115e-06*tu05
				- 4.948996960052575e-08*tu06
				- 2.345390965359666e-09*tu07
				+ 6.710522628543514e-11*tu08
				+ 1.979867116023822e-12*tu09
				- 1.027163441987459e-13*tu10
				- 1.836436639605094e-15*tu11
				+ 1.633620651699784e-16*tu12
				- 6.931471805599453e-01*HY3(0,0,-1);
			HY4(0,0,1,-1) =
				- 4.713463351559199e-02
				- 3.918037828258655e-01*tv01
				- 4.730698763577787e-02*tv02
				- 3.532784273601097e-03*tv03
				- 1.724036773635937e-04*tv04
				- 5.100573466380115e-06*tv05
				- 4.948996960052575e-08*tv06
				+ 2.345390965359666e-09*tv07
				+ 6.710522628543514e-11*tv08
				- 1.979867116023822e-12*tv09
				- 1.027163441987459e-13*tv10
				+ 1.836436639605094e-15*tv11
				+ 1.633620651699784e-16*tv12
				+ 6.931471805599453e-01*HY3(0,0,1);
			HY4(0,-1,0,1) =
				- 5.610575179941452e-02
				+ 4.649892609082033e-01*tu01
				- 5.631239161843284e-02*tu02
				+ 4.220972769653239e-03*tu03
				- 2.066940413626322e-04*tu04
				+ 6.100628682175971e-06*tu05
				- 5.412969106099992e-08*tu06
				- 3.230915912784154e-09*tu07
				+ 9.249866333323043e-11*tu08
				+ 2.685990764581699e-12*tu09
				- 1.543312114608473e-13*tu10
				- 2.036971731594398e-15*tu11
				+ 2.517450307574790e-16*tu12
				- 8.224670334241132e-01*HY2(0,-1);
			HY4(0,-1,-1,1) =
				- 4.031271939759038e-02
				+ 3.295217254379970e-01*tu01
				- 4.047097737450547e-02*tu02
				+ 3.104955391145708e-03*tu03
				- 1.583251510732719e-04*tu04
				+ 5.083334568184305e-06*tu05
				- 6.708598619683341e-08*tu06
				- 1.944278941559733e-09*tu07
				+ 8.804863765356287e-11*tu08
				+ 9.341312729419985e-13*tu09
				- 1.231746977889946e-13*tu10
				+ 3.370647349658755e-16*tu11
				+ 1.718647072955689e-16*tu12
				- 5.822405264650125e-01*HY2(0,-1)
				- 6.931471805599453e-01*HY3(0,-1,-1);
			HY4(0,-1,1,-1) =
				- 4.495764739674318e-02
				- 2.758514579198452e-01*tv01
				- 4.515130668959398e-02*tv02
				- 3.875995092451054e-03*tv03
				- 1.936768370518385e-04*tv04
				- 5.133195476137788e-06*tv05
				- 1.752786900562004e-08*tv06
				+ 2.715518363893619e-09*tv07
				+ 1.631155670579918e-11*tv08
				- 2.940721244025822e-12*tv09
				- 2.045219059123054e-14*tv10
				+ 3.895696592051861e-15*tv11
				+ 4.804530139182014e-01*HY2(0,-1)
				+ 6.931471805599453e-01*HY3(0,-1,1);
			HY4(0,1,-1,-1) =
				- 2.782664607935622e-02
				- 1.410831481728889e-01*tv01
				- 2.801876266982354e-02*tv02
				- 2.997894208020603e-03*tv03
				- 1.921960113936824e-04*tv04
				- 7.016503666427137e-06*tv05
				- 7.928257765061337e-08*tv06
				+ 4.388745575295455e-09*tv07
				+ 1.381107719492586e-10*tv08
				- 4.341921500497716e-12*tv09
				- 2.375364913875066e-13*tv10
				+ 4.522044546598701e-15*tv11
				+ 4.033357472727688e-16*tv12
				+ 2.402265069591007e-01*HY2(0,1);
			HY4(0,-1,1,1) =
				+ 2.782664607935622e-02
				- 1.410831481728889e-01*tu01
				+ 2.801876266982354e-02*tu02
				- 2.997894208020603e-03*tu03
				+ 1.921960113936824e-04*tu04
				- 7.016503666427137e-06*tu05
				+ 7.928257765061337e-08*tu06
				+ 4.388745575295455e-09*tu07
				- 1.381107719492586e-10*tu08
				- 4.341921500497716e-12*tu09
				+ 2.375364913875066e-13*tu10
				+ 4.522044546598701e-15*tu11
				- 4.033357472727688e-16*tu12
				+ 2.402265069591007e-01*HY2(0,-1);
			HY4(0,1,-1,1) =
				+ 4.495764739674318e-02
				- 2.758514579198452e-01*tu01
				+ 4.515130668959398e-02*tu02
				- 3.875995092451054e-03*tu03
				+ 1.936768370518385e-04*tu04
				- 5.133195476137788e-06*tu05
				+ 1.752786900562004e-08*tu06
				+ 2.715518363893619e-09*tu07
				- 1.631155670579918e-11*tu08
				- 2.940721244025822e-12*tu09
				+ 2.045219059123054e-14*tu10
				+ 3.895696592051861e-15*tu11
				+ 4.804530139182014e-01*HY2(0,1)
				- 6.931471805599453e-01*HY3(0,1,-1);
			HY4(0,1,1,-1) =
				+ 4.031271939759038e-02
				+ 3.295217254379970e-01*tv01
				+ 4.047097737450547e-02*tv02
				+ 3.104955391145708e-03*tv03
				+ 1.583251510732719e-04*tv04
				+ 5.083334568184305e-06*tv05
				+ 6.708598619683341e-08*tv06
				- 1.944278941559733e-09*tv07
				- 8.804863765356287e-11*tv08
				+ 9.341312729419985e-13*tv09
				+ 1.231746977889946e-13*tv10
				+ 3.370647349658755e-16*tv11
				- 1.718647072955689e-16*tv12
				- 5.822405264650125e-01*HY2(0,1)
				+ 6.931471805599453e-01*HY3(0,1,1);
			HY4(-1,-1,-1,1) =
				- 3.768651335815766e-02
				+ 3.043162147119780e-01*tu01
				- 3.784162844891144e-02*tu02
				+ 2.958351024362477e-03*tu03
				- 1.551924666783514e-04*tu04
				+ 5.216293832777793e-06*tu05
				- 7.726843592398867e-08*tu06
				- 1.910379383726989e-09*tu07
				+ 1.073377838077624e-10*tu08
				+ 4.147979000313175e-13*tu09
				- 1.506593045440627e-13*tu10
				+ 1.921276747438603e-15*tu11
				+ 1.977332880766160e-16*tu12
				- 5.372131936080402e-01*HY1(-1)
				- 2.911202632325062e-01*HY1(-1)*HY1(-1)
				- 1.155245300933242e-01*HY1(-1)*HY1(-1)*HY1(-1);
			HY4(-1,-1,1,1) =
				+ 2.908893189635991e-02
				- 1.784837106345115e-01*tu01
				+ 2.927117884632272e-02*tu02
				- 2.888221776586007e-03*tu03
				+ 1.823501630828519e-04*tu04
				- 6.976883920991888e-06*tu05
				+ 1.030302948541690e-07*tu06
				+ 3.794029548474434e-09*tu07
				- 1.825184393299693e-10*tu08
				- 2.300206200729610e-12*tu09
				+ 3.062629564489397e-13*tu10
				- 7.629393984387632e-16*tu11
				- 4.860728618463296e-16*tu12
				+ 3.088253750968339e-01*HY1(-1)
				+ 1.201132534795503e-01*HY1(-1)*HY1(-1);
			HY4(-1,1,1,1) =
				- 9.029205146496301e-03
				+ 3.753824045412342e-02*tu01
				- 9.240717745810759e-03*tu02
				+ 2.351153976182453e-03*tu03
				- 2.115782190216214e-04*tu04
				+ 8.486524807740892e-06*tu05
				- 6.547885807612483e-08*tu06
				- 6.934422754020238e-09*tu07
				+ 1.405695202725693e-10*tu08
				+ 8.329441237576153e-12*tu09
				- 2.790404594803712e-13*tu10
				- 1.024489568815216e-14*tu11
				+ 5.256388245544115e-16*tu12
				- 5.550410866482157e-02*HY1(-1);
		} // nw > 3 endif
	} // (n1,n2) = (-1,1) -- completion endif
	return;
}


/**
 *
 * @brief evaluates the HPL for r2m1 < y < r2p1
 * fillirr1dhplat1 is called by eval1dhplat1 after calling
 * fillirr1dhplat0 with argument r=(1-y)/(1+y)
 * it is guaranteed that nw is in the range 2:4, and that (n1,n2)
 * take one of the pairs of values (0,1), (-1,0) or (-1,1)
 */
void hpl::details::fillirr1dhplat1(double r, int nw,
		FA1dD11& HR1, FA2dD11& HR2, FA3dD11& HR3, FA4dD11& HR4,
		FA1dD11& HY1, FA2dD11& HY2, FA3dD11& HY3, FA4dD11& HY4,
		FA1dD11& Hi1, FA2dD11& Hi2, FA3dD11& Hi3, FA4dD11& Hi4,
		int n1, int n2)
{
	// (n1,n2) = (0,1) or (-1,1)
	if ( ( (n1==0) && (n2==1) ) || ( (n1==-1) && (n2==1) ) )
	{
		HY2(0,1) =
			+ 1.6449340668482264e+00
			+ 6.9314718055994530e-01*HR1(-1)
			- 5.0000000000000000e-01*HR1(-1)*HR1(-1)
			+ HR1( -1)*HR1(0)
			- HR1( -1)*HR1(1)
			+ HR1(0) *HR1(1)
			+ 6.9314718055994530e-01*HR1(1)
			+ HR2( -1,1)
			- HR2(0, -1)
			- HR2(0,1);
		if (r<0.)
		{
			Hi2(0,1) = - HR1( -1) - HR1(1);
		}
		if ( nw>2 )
		{
			HY3(0,0,1) =
				+ 1.2020569031595942e+00
				- 1.6449340668482264e+00*HR1(-1)
				- 3.4657359027997265e-01*HR1(-1)*HR1(-1)
				+ 1.6666666666666666e-01*HR1(-1)*HR1(-1)*HR1(-1)
				- 5.0000000000000000e-01*HR1(-1)*HR1(-1)*HR1(0)
				+ 5.0000000000000000e-01*HR1(-1)*HR1(-1)*HR1(1)
				- HR1( -1)*HR1(0)*HR1(1)
				- 6.9314718055994530e-01*HR1(-1)*HR1(1)
				+ 5.0000000000000000e-01*HR1(-1)*HR1(1)*HR1(1)
				+ HR1( -1)*HR2(0,-1)
				+ HR1( -1)*HR2(0,1)
				- 5.0000000000000000e-01*HR1(0)*HR1(1)*HR1(1)
				- 1.6449340668482264e+00*HR1(1)
				- 3.4657359027997265e-01*HR1(1)*HR1(1)
				- HR1(1) *HR2(-1,1)
				+ HR1(1) *HR2(0,-1)
				+ HR1(1) *HR2(0,1)
				- HR3( -1,-1,1)
				+ HR3( -1,1,1)
				- HR3(0, -1,-1)
				- HR3(0, -1,1)
				- HR3(0,1, -1)
				- HR3(0,1,1);
			HY3(0,1,1) =
				+ 1.2020569031595942e+00
				- 2.4022650695910071e-01*HR1(-1)
				+ 3.4657359027997265e-01*HR1(-1)*HR1(-1)
				- 1.6666666666666666e-01*HR1(-1)*HR1(-1)*HR1(-1)
				+ 5.0000000000000000e-01*HR1(-1)*HR1(-1)*HR1(0)
				- 5.0000000000000000e-01*HR1(-1)*HR1(-1)*HR1(1)
				- 6.9314718055994530e-01*HR1(-1)*HR1(0)
				- 5.0000000000000000e-01*HR1(-1)*HR1(0)*HR1(0)
				+ HR1( -1)*HR1(0)*HR1(1)
				+ 6.9314718055994530e-01*HR1(-1)*HR1(1)
				+ HR1( -1)*HR2(-1,1)
				- 5.0000000000000000e-01*HR1(0)*HR1(0)*HR1(1)
				- 6.9314718055994530e-01*HR1(0)*HR1(1)
				- HR1(0) *HR2(-1,1)
				+ HR1(0) *HR2(0,-1)
				+ HR1(0) *HR2(0,1)
				- 2.4022650695910071e-01*HR1(1)
				- 6.9314718055994530e-01*HR2(-1,1)
				+ 6.9314718055994530e-01*HR2(0,-1)
				+ 6.9314718055994530e-01*HR2(0,1)
				- HR3( -1,-1,1)
				- HR3(0, -1,-1)
				- HR3(0,0, -1)
				- HR3(0,0,1)
				- HR3(0,1, -1);
			if (r<0.)
			{
				HY3(0,1,1) = HY3(0,1,1)
						+ 4.9348022005446793e+00*HR1(-1)
						+ 4.9348022005446793e+00*HR1(1);
				Hi3(0,0,1) =
					+ 5.0000000000000000e-01*HR1(-1)*HR1(-1)
					+ HR1( -1)*HR1(1)
					+ 5.0000000000000000e-01*HR1(1)*HR1(1);
				Hi3(0,1,1) =
					+ 6.9314718055994530e-01*HR1(-1)
					- 5.0000000000000000e-01*HR1(-1)*HR1(-1)
					+ HR1( -1)*HR1(0)
					- HR1( -1)*HR1(1)
					+ HR1(0) *HR1(1)
					+ 6.9314718055994530e-01*HR1(1)
					+ HR2( -1,1)
					- HR2(0, -1)
					- HR2(0,1);
			}
		}
		if ( nw>3 )
		{
			HY4(0,0,0,1) =
				+ 1.0823232337111381e+00
				- 1.2020569031595942e+00*HR1(-1)
				+ 8.2246703342411321e-01*HR1(-1)*HR1(-1)
				+ 1.1552453009332421e-01*HR1(-1)*HR1(-1)*HR1(-1)
				- 4.1666666666666666e-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
				+ 1.6666666666666666e-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
				- 1.6666666666666666e-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
				+ 5.0000000000000000e-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
				+ 3.4657359027997265e-01*HR1(-1)*HR1(-1)*HR1(1)
				- 2.5000000000000000e-01*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
				- 5.0000000000000000e-01*HR1(-1)*HR1(-1)*HR2(0,-1)
				- 5.0000000000000000e-01*HR1(-1)*HR1(-1)*HR2(0,1)
				+ 5.0000000000000000e-01*HR1(-1)*HR1(0)*HR1(1)*HR1(1)
				+ 1.6449340668482264e+00*HR1(-1)*HR1(1)
				+ 3.4657359027997265e-01*HR1(-1)*HR1(1)*HR1(1)
				- 1.6666666666666666e-01*HR1(-1)*HR1(1)*HR1(1)*HR1(1)
				- HR1( -1)*HR1(1)*HR2(0,-1)
				- HR1( -1)*HR1(1)*HR2(0,1)
				+ HR1( -1)*HR3(0,-1,-1)
				+ HR1( -1)*HR3(0,-1,1)
				+ HR1( -1)*HR3(0,1,-1)
				+ HR1( -1)*HR3(0,1,1)
				+ 1.6666666666666666e-01*HR1(0)*HR1(1)*HR1(1)*HR1(1)
				- 1.2020569031595942e+00*HR1(1)
				+ 8.2246703342411321e-01*HR1(1)*HR1(1)
				+ 1.1552453009332421e-01*HR1(1)*HR1(1)*HR1(1)
				+ 5.0000000000000000e-01*HR1(1)*HR1(1)*HR2(-1,1)
				- 5.0000000000000000e-01*HR1(1)*HR1(1)*HR2(0,-1)
				- 5.0000000000000000e-01*HR1(1)*HR1(1)*HR2(0,1)
				+ HR1(1) *HR3(-1,-1,1)
				- HR1(1) *HR3(-1,1,1)
				+ HR1(1) *HR3(0,-1,-1)
				+ HR1(1) *HR3(0,-1,1)
				+ HR1(1) *HR3(0,1,-1)
				+ HR1(1) *HR3(0,1,1)
				+ HR4( -1,-1,-1,1)
				- HR4( -1,-1,1,1)
				+ HR4( -1,1,1,1)
				- HR4(0, -1,-1,-1)
				- HR4(0, -1,-1,1)
				- HR4(0, -1,1,-1)
				- HR4(0, -1,1,1)
				- HR4(0,1, -1,-1)
				- HR4(0,1, -1,1)
				- HR4(0,1,1, -1)
				- HR4(0,1,1,1);
			HY4(0,0,1,1) =
				+ 2.7058080842778454e-01
				- 1.2020569031595942e+00*HR1(-1)
				+ 1.2011325347955035e-01*HR1(-1)*HR1(-1)
				- 1.1552453009332421e-01*HR1(-1)*HR1(-1)*HR1(-1)
				+ 4.1666666666666666e-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
				- 1.6666666666666666e-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
				+ 1.6666666666666666e-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
				+ 3.4657359027997265e-01*HR1(-1)*HR1(-1)*HR1(0)
				+ 2.5000000000000000e-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)
				- 5.0000000000000000e-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
				- 3.4657359027997265e-01*HR1(-1)*HR1(-1)*HR1(1)
				+ 2.5000000000000000e-01*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
				+ 5.0000000000000000e-01*HR1(-1)*HR1(0)*HR1(0)*HR1(1)
				+ 6.9314718055994530e-01*HR1(-1)*HR1(0)*HR1(1)
				- 5.0000000000000000e-01*HR1(-1)*HR1(0)*HR1(1)*HR1(1)
				- HR1( -1)*HR1(0)*HR2(0,-1)
				- HR1( -1)*HR1(0)*HR2(0,1)
				+ 2.4022650695910071e-01*HR1(-1)*HR1(1)
				- 3.4657359027997265e-01*HR1(-1)*HR1(1)*HR1(1)
				- HR1( -1)*HR1(1)*HR2(-1,1)
				- 6.9314718055994530e-01*HR1(-1)*HR2(0,-1)
				- 6.9314718055994530e-01*HR1(-1)*HR2(0,1)
				- HR1( -1)*HR3(-1,-1,1)
				+ HR1( -1)*HR3(-1,1,1)
				+ HR1( -1)*HR3(0,-1,-1)
				+ HR1( -1)*HR3(0,0,-1)
				+ HR1( -1)*HR3(0,0,1)
				+ HR1( -1)*HR3(0,1,-1)
				+ 2.5000000000000000e-01*HR1(0)*HR1(0)*HR1(1)*HR1(1)
				+ 3.4657359027997265e-01*HR1(0)*HR1(1)*HR1(1)
				+ HR1(0) *HR1(1)*HR2(-1,1)
				- HR1(0) *HR1(1)*HR2(0,-1)
				- HR1(0) *HR1(1)*HR2(0,1)
				+ HR1(0) *HR3(-1,-1,1)
				- HR1(0) *HR3(-1,1,1)
				+ HR1(0) *HR3(0,-1,-1)
				+ HR1(0) *HR3(0,-1,1)
				+ HR1(0) *HR3(0,1,-1)
				+ HR1(0) *HR3(0,1,1)
				- 1.2020569031595942e+00*HR1(1)
				+ 1.2011325347955035e-01*HR1(1)*HR1(1)
				+ 6.9314718055994530e-01*HR1(1)*HR2(-1,1)
				- 6.9314718055994530e-01*HR1(1)*HR2(0,-1)
				- 6.9314718055994530e-01*HR1(1)*HR2(0,1)
				+ HR1(1) *HR3(-1,-1,1)
				+ HR1(1) *HR3(0,-1,-1)
				+ HR1(1) *HR3(0,0,-1)
				+ HR1(1) *HR3(0,0,1)
				+ HR1(1) *HR3(0,1,-1)
				+ 6.9314718055994530e-01*HR3(-1,-1,1)
				- 6.9314718055994530e-01*HR3(-1,1,1)
				+ 6.9314718055994530e-01*HR3(0,-1,-1)
				+ 6.9314718055994530e-01*HR3(0,-1,1)
				+ 6.9314718055994530e-01*HR3(0,1,-1)
				+ 6.9314718055994530e-01*HR3(0,1,1)
				+ 2.0000000000000000e+00*HR4(-1,-1,-1,1)
				- HR4( -1,-1,1,1)
				- 2.0000000000000000e+00*HR4(0,-1,-1,-1)
				- HR4(0, -1,-1,1)
				- HR4(0, -1,1,-1)
				- HR4(0,0, -1,-1)
				- HR4(0,0, -1,1)
				- HR4(0,0,1, -1)
				- HR4(0,0,1,1)
				- 2.0000000000000000e+00*HR4(0,1,-1,-1)
				- HR4(0,1, -1,1)
				- HR4(0,1,1, -1);
			HY4(0,1,1,1) =
				+ 1.0823232337111381e+00
				+ 5.5504108664821579e-02*HR1(-1)
				- 1.2011325347955035e-01*HR1(-1)*HR1(-1)
				+ 1.1552453009332421e-01*HR1(-1)*HR1(-1)*HR1(-1)
				- 4.1666666666666666e-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
				+ 1.6666666666666666e-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
				- 1.6666666666666666e-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
				- 3.4657359027997265e-01*HR1(-1)*HR1(-1)*HR1(0)
				- 2.5000000000000000e-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)
				+ 5.0000000000000000e-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
				+ 3.4657359027997265e-01*HR1(-1)*HR1(-1)*HR1(1)
				+ 5.0000000000000000e-01*HR1(-1)*HR1(-1)*HR2(-1,1)
				+ 2.4022650695910071e-01*HR1(-1)*HR1(0)
				+ 3.4657359027997265e-01*HR1(-1)*HR1(0)*HR1(0)
				+ 1.6666666666666666e-01*HR1(-1)*HR1(0)*HR1(0)*HR1(0)
				- 5.0000000000000000e-01*HR1(-1)*HR1(0)*HR1(0)*HR1(1)
				- 6.9314718055994530e-01*HR1(-1)*HR1(0)*HR1(1)
				- HR1( -1)*HR1(0)*HR2(-1,1)
				- 2.4022650695910071e-01*HR1(-1)*HR1(1)
				- 6.9314718055994530e-01*HR1(-1)*HR2(-1,1)
				- HR1( -1)*HR3(-1,-1,1)
				+ 1.6666666666666666e-01*HR1(0)*HR1(0)*HR1(0)*HR1(1)
				+ 3.4657359027997265e-01*HR1(0)*HR1(0)*HR1(1)
				+ 5.0000000000000000e-01*HR1(0)*HR1(0)*HR2(-1,1)
				- 5.0000000000000000e-01*HR1(0)*HR1(0)*HR2(0,-1)
				- 5.0000000000000000e-01*HR1(0)*HR1(0)*HR2(0,1)
				+ 2.4022650695910071e-01*HR1(0)*HR1(1)
				+ 6.9314718055994530e-01*HR1(0)*HR2(-1,1)
				- 6.9314718055994530e-01*HR1(0)*HR2(0,-1)
				- 6.9314718055994530e-01*HR1(0)*HR2(0,1)
				+ HR1(0) *HR3(-1,-1,1)
				+ HR1(0) *HR3(0,-1,-1)
				+ HR1(0) *HR3(0,0,-1)
				+ HR1(0) *HR3(0,0,1)
				+ HR1(0) *HR3(0,1,-1)
				+ 5.5504108664821579e-02*HR1(1)
				+ 2.4022650695910071e-01*HR2(-1,1)
				- 2.4022650695910071e-01*HR2(0,-1)
				- 2.4022650695910071e-01*HR2(0,1)
				+ 6.9314718055994530e-01*HR3(-1,-1,1)
				+ 6.9314718055994530e-01*HR3(0,-1,-1)
				+ 6.9314718055994530e-01*HR3(0,0,-1)
				+ 6.9314718055994530e-01*HR3(0,0,1)
				+ 6.9314718055994530e-01*HR3(0,1,-1)
				+ HR4( -1,-1,-1,1)
				- HR4(0, -1,-1,-1)
				- HR4(0,0, -1,-1)
				- HR4(0,0,0, -1)
				- HR4(0,0,0,1)
				- HR4(0,0,1, -1)
				- HR4(0,1, -1,-1);
			if (r<0.)
			{
				HY4(0,0,1,1) = HY4(0,0,1,1)
					- 2.4674011002723396e+00*HR1(-1)*HR1(-1)
					- 4.9348022005446793e+00*HR1(-1)*HR1(1)
					- 2.4674011002723396e+00*HR1(1)*HR1(1);
				HY4(0,1,1,1) = HY4(0,1,1,1)
					- 3.4205442319285582e+00*HR1(-1)
					+ 2.4674011002723396e+00*HR1(-1)*HR1(-1)
					- 4.9348022005446793e+00*HR1(-1)*HR1(0)
					+ 4.9348022005446793e+00*HR1(-1)*HR1(1)
					- 4.9348022005446793e+00*HR1(0)*HR1(1)
					- 3.4205442319285582e+00*HR1(1)
					- 4.9348022005446793e+00*HR2(-1,1)
					+ 4.9348022005446793e+00*HR2(0,-1)
					+ 4.9348022005446793e+00*HR2(0,1);
				Hi4(0,0,0,1) =
			   - 1.666666666666666e-01*HR1(-1)*HR1(-1)*HR1(-1)
			   - 5.000000000000000e-01*HR1(-1)*HR1(-1)*HR1(1)
			   - 5.000000000000000e-01*HR1(-1)*HR1(1)*HR1(1)
			   - 1.666666666666666e-01*HR1(1)*HR1(1)*HR1(1);
				Hi4(0,0,1,1) =
					- 3.465735902799726e-01*HR1(-1)*HR1(-1)
					+ 1.666666666666666e-01*HR1(-1)*HR1(-1)*HR1(-1)
					- 5.000000000000000e-01*HR1(-1)*HR1(-1)*HR1(0)
					+ 5.000000000000000e-01*HR1(-1)*HR1(-1)*HR1(1)
					- HR1( -1)*HR1(0)*HR1(1)
					- 6.931471805599453e-01*HR1(-1)*HR1(1)
					+ 5.000000000000000e-01*HR1(-1)*HR1(1)*HR1(1)
					+ HR1( -1)*HR2(0,-1)
					+ HR1( -1)*HR2(0,1)
					- 5.000000000000000e-01*HR1(0)*HR1(1)*HR1(1)
					- 3.465735902799726e-01*HR1(1)*HR1(1)
					- HR1(1) *HR2(-1,1)
					+ HR1(1) *HR2(0,-1)
					+ HR1(1) *HR2(0,1)
					- HR3( -1,-1,1)
					+ HR3( -1,1,1)
					- HR3(0, -1,-1)
					- HR3(0, -1,1)
					- HR3(0,1, -1)
					- HR3(0,1,1);
				Hi4(0,1,1,1) =
					+ 1.404707559889125e+00*HR1(-1)
					+ 3.465735902799726e-01*HR1(-1)*HR1(-1)
					- 1.666666666666666e-01*HR1(-1)*HR1(-1)*HR1(-1)
					+ 5.000000000000000e-01*HR1(-1)*HR1(-1)*HR1(0)
					- 5.000000000000000e-01*HR1(-1)*HR1(-1)*HR1(1)
					- 6.931471805599453e-01*HR1(-1)*HR1(0)
					- 5.000000000000000e-01*HR1(-1)*HR1(0)*HR1(0)
					+ HR1( -1)*HR1(0)*HR1(1)
					+ 6.931471805599453e-01*HR1(-1)*HR1(1)
					+ HR1( -1)*HR2(-1,1)
					- 5.000000000000000e-01*HR1(0)*HR1(0)*HR1(1)
					- 6.931471805599453e-01*HR1(0)*HR1(1)
					- HR1(0) *HR2(-1,1)
					+ HR1(0) *HR2(0,-1)
					+ HR1(0) *HR2(0,1)
					+ 1.404707559889125e+00*HR1(1)
					- 6.931471805599453e-01*HR2(-1,1)
					+ 6.931471805599453e-01*HR2(0,-1)
					+ 6.931471805599453e-01*HR2(0,1)
					- HR3( -1,-1,1)
					- HR3(0, -1,-1)
					- HR3(0,0, -1)
					- HR3(0,0,1)
					- HR3(0,1, -1);
			}
		} // nw > 3 endif
	} // (n1,n2) = (0,1) or (-1,1) endif

	//************
	// (n1,n2) = (-1,0) or (-1,1)
	if ( ( (n1==-1) && (n2==0) ) || ( (n1==-1) && (n2==1) ) )
	{
		HY2(0,-1) =
			+ 8.2246703342411321e-01
			- 6.9314718055994530e-01*HR1(-1)
			+ 5.0000000000000000e-01*HR1(-1)*HR1(-1)
			+ HR1( -1)*HR1(1)
			- 6.9314718055994530e-01*HR1(1)
			- HR2( -1,1);
		if ( nw>2 )
		{
			HY3(0,0,-1) =
				+ 9.0154267736969571e-01
				- 8.2246703342411321e-01*HR1(-1)
				+ 3.4657359027997265e-01*HR1(-1)*HR1(-1)
				- 1.6666666666666666e-01*HR1(-1)*HR1(-1)*HR1(-1)
				- 5.0000000000000000e-01*HR1(-1)*HR1(-1)*HR1(1)
				+ 6.9314718055994530e-01*HR1(-1)*HR1(1)
				- 5.0000000000000000e-01*HR1(-1)*HR1(1)*HR1(1)
				- 8.2246703342411321e-01*HR1(1)
				+ 3.4657359027997265e-01*HR1(1)*HR1(1)
				+ HR1(1) *HR2(-1,1)
				+ HR3( -1,-1,1)
				- HR3( -1,1,1);
			HY3(0,-1,-1) =
				+ 1.5025711289494928e-01
				- 2.4022650695910071e-01*HR1(-1)
				+ 3.4657359027997265e-01*HR1(-1)*HR1(-1)
				- 1.6666666666666666e-01*HR1(-1)*HR1(-1)*HR1(-1)
				- 5.0000000000000000e-01*HR1(-1)*HR1(-1)*HR1(1)
				+ 6.9314718055994530e-01*HR1(-1)*HR1(1)
				+ HR1( -1)*HR2(-1,1)
				- 2.4022650695910071e-01*HR1(1)
				- 6.9314718055994530e-01*HR2(-1,1)
				- HR3( -1,-1,1);
		}
		if ( nw>3 )
		{
			HY4(0,0,0,-1) =
				+ 9.4703282949724591e-01
				- 9.0154267736969571e-01*HR1(-1)
				+ 4.1123351671205660e-01*HR1(-1)*HR1(-1)
				- 1.1552453009332421e-01*HR1(-1)*HR1(-1)*HR1(-1)
				+ 4.1666666666666666e-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
				+ 1.6666666666666666e-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
				- 3.4657359027997265e-01*HR1(-1)*HR1(-1)*HR1(1)
				+ 2.5000000000000000e-01*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
				+ 8.2246703342411321e-01*HR1(-1)*HR1(1)
				- 3.4657359027997265e-01*HR1(-1)*HR1(1)*HR1(1)
				+ 1.6666666666666666e-01*HR1(-1)*HR1(1)*HR1(1)*HR1(1)
				- 9.0154267736969571e-01*HR1(1)
				+ 4.1123351671205660e-01*HR1(1)*HR1(1)
				- 1.1552453009332421e-01*HR1(1)*HR1(1)*HR1(1)
				- 5.0000000000000000e-01*HR1(1)*HR1(1)*HR2(-1,1)
				- HR1(1) *HR3(-1,-1,1)
				+ HR1(1) *HR3(-1,1,1)
				- HR4( -1,-1,-1,1)
				+ HR4( -1,-1,1,1)
				- HR4( -1,1,1,1);
			HY4(0,0,-1,-1) =
				+ 8.7785671568655302e-02
				- 1.5025711289494928e-01*HR1(-1)
				+ 1.2011325347955035e-01*HR1(-1)*HR1(-1)
				- 1.1552453009332421e-01*HR1(-1)*HR1(-1)*HR1(-1)
				+ 4.1666666666666666e-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
				+ 1.6666666666666666e-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
				- 3.4657359027997265e-01*HR1(-1)*HR1(-1)*HR1(1)
				+ 2.5000000000000000e-01*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
				+ 2.4022650695910071e-01*HR1(-1)*HR1(1)
				- 3.4657359027997265e-01*HR1(-1)*HR1(1)*HR1(1)
				- HR1( -1)*HR1(1)*HR2(-1,1)
				- HR1( -1)*HR3(-1,-1,1)
				+ HR1( -1)*HR3(-1,1,1)
				- 1.5025711289494928e-01*HR1(1)
				+ 1.2011325347955035e-01*HR1(1)*HR1(1)
				+ 6.9314718055994530e-01*HR1(1)*HR2(-1,1)
				+ HR1(1) *HR3(-1,-1,1)
				+ 6.9314718055994530e-01*HR3(-1,-1,1)
				- 6.9314718055994530e-01*HR3(-1,1,1)
				+ 2.0000000000000000e+00*HR4(-1,-1,-1,1)
				- HR4( -1,-1,1,1);
			HY4(0,-1,-1,-1) =
				+ 2.3752366322618485e-02
				- 5.5504108664821579e-02*HR1(-1)
				+ 1.2011325347955035e-01*HR1(-1)*HR1(-1)
				- 1.1552453009332421e-01*HR1(-1)*HR1(-1)*HR1(-1)
				+ 4.1666666666666666e-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
				+ 1.6666666666666666e-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
				- 3.4657359027997265e-01*HR1(-1)*HR1(-1)*HR1(1)
				- 5.0000000000000000e-01*HR1(-1)*HR1(-1)*HR2(-1,1)
				+ 2.4022650695910071e-01*HR1(-1)*HR1(1)
				+ 6.9314718055994530e-01*HR1(-1)*HR2(-1,1)
				+ HR1( -1)*HR3(-1,-1,1)
				- 5.5504108664821579e-02*HR1(1)
				- 2.4022650695910071e-01*HR2(-1,1)
				- 6.9314718055994530e-01*HR3(-1,-1,1)
				- HR4( -1,-1,-1,1);
		} // nw > 3 endif
	} // (n1,n2) = (-1,0) or (-1,1) endif
	// (n1,n2) = (-1,1) -- completion
	if ( (n1==-1) && (n2==1) )
	{
		HY2(-1,1) =
			+ 5.8224052646501250e-01
			+ 6.9314718055994530e-01*HR1(-1)
			- 5.0000000000000000e-01*HR1(-1)*HR1(-1)
			+ HR1( -1)*HR1(0)
			- HR2(0, -1);
		if (r<0.)
		{
			Hi2(-1,1) = - HR1( -1);
		}
		if ( nw>2 )
		{
			HY3(0,-1,1) =
				+ 2.4307035167006157e-01
				- 5.8224052646501250e-01*HR1(-1)
				- 3.4657359027997265e-01*HR1(-1)*HR1(-1)
				+ 1.6666666666666666e-01*HR1(-1)*HR1(-1)*HR1(-1)
				- 5.0000000000000000e-01*HR1(-1)*HR1(-1)*HR1(0)
				+ 5.0000000000000000e-01*HR1(-1)*HR1(-1)*HR1(1)
				- HR1( -1)*HR1(0)*HR1(1)
				- 6.9314718055994530e-01*HR1(-1)*HR1(1)
				- HR1( -1)*HR2(-1,1)
				+ HR1( -1)*HR2(0,-1)
				+ HR1(0) *HR2(-1,1)
				- 5.8224052646501250e-01*HR1(1)
				+ HR1(1) *HR2(0,-1)
				+ 6.9314718055994530e-01*HR2(-1,1)
				+ HR3( -1,-1,1)
				- HR3(0, -1,-1)
				- HR3(0, -1,1);
			HY3(0,1,-1) =
				+ 5.0821521280468485e-01
				+ 1.0626935403832139e+00*HR1(-1)
				- 3.4657359027997265e-01*HR1(-1)*HR1(-1)
				+ 1.6666666666666666e-01*HR1(-1)*HR1(-1)*HR1(-1)
				+ 5.0000000000000000e-01*HR1(-1)*HR1(-1)*HR1(1)
				+ 6.9314718055994530e-01*HR1(-1)*HR1(0)
				- 6.9314718055994530e-01*HR1(-1)*HR1(1)
				- HR1( -1)*HR2(-1,1)
				- HR1( -1)*HR2(0,-1)
				+ 6.9314718055994530e-01*HR1(0)*HR1(1)
				+ 1.0626935403832139e+00*HR1(1)
				- HR1(1) *HR2(0,-1)
				+ 6.9314718055994530e-01*HR2(-1,1)
				- 6.9314718055994530e-01*HR2(0,-1)
				- 6.9314718055994530e-01*HR2(0,1)
				+ HR3( -1,-1,1)
				+ 2.0000000000000000e+00*HR3(0,-1,-1)
				+ HR3(0, -1,1)
				+ HR3(0,1, -1);
			HY3(-1,-1,1) =
				+ 9.4753004230127705e-02
				- 5.8224052646501250e-01*HR1(-1)
				- 3.4657359027997265e-01*HR1(-1)*HR1(-1)
				+ 1.6666666666666666e-01*HR1(-1)*HR1(-1)*HR1(-1)
				- 5.0000000000000000e-01*HR1(-1)*HR1(-1)*HR1(0)
				+ HR1( -1)*HR2(0,-1)
				- HR3(0, -1,-1);
			HY3(-1,1,1) =
				+ 5.3721319360804020e-01
				- 2.4022650695910071e-01*HR1(-1)
				+ 3.4657359027997265e-01*HR1(-1)*HR1(-1)
				- 1.6666666666666666e-01*HR1(-1)*HR1(-1)*HR1(-1)
				+ 5.0000000000000000e-01*HR1(-1)*HR1(-1)*HR1(0)
				- 6.9314718055994530e-01*HR1(-1)*HR1(0)
				- 5.0000000000000000e-01*HR1(-1)*HR1(0)*HR1(0)
				+ HR1(0) *HR2(0,-1)
				+ 6.9314718055994530e-01*HR2(0,-1)
				- HR3(0, -1,-1)
				- HR3(0,0, -1);
			if (r<0.)
			{
				HY3(-1,1,1) = HY3(-1,1,1) + 4.9348022005446793e+00*HR1(-1);
				Hi3(0,-1,1) =
					+ 5.0000000000000000e-01*HR1(-1)*HR1(-1)
					+ HR1( -1)*HR1(1)
					- HR2( -1,1);
				Hi3(0,1,-1) =
					- 6.9314718055994530e-01*HR1(-1)
					- 6.9314718055994530e-01*HR1(1);
				Hi3(-1,-1,1) = 5.0000000000000000e-01*HR1(-1)*HR1(-1);
				Hi3(-1,1,1) =
					+ 6.9314718055994530e-01*HR1(-1)
					- 5.0000000000000000e-01*HR1(-1)*HR1(-1)
					+ HR1( -1)*HR1(0)
					- HR2(0, -1);
			}
		}
		if ( nw>3 )
		{
			HY4(0,0,-1,1) =
				+ 1.1787599965050932e-01
				- 2.4307035167006157e-01*HR1(-1)
				+ 2.9112026323250625e-01*HR1(-1)*HR1(-1)
				+ 1.1552453009332421e-01*HR1(-1)*HR1(-1)*HR1(-1)
				- 4.1666666666666666e-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
				+ 1.6666666666666666e-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
				- 1.6666666666666666e-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
				+ 5.0000000000000000e-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
				+ 3.4657359027997265e-01*HR1(-1)*HR1(-1)*HR1(1)
				- 2.5000000000000000e-01*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
				- 5.0000000000000000e-01*HR1(-1)*HR1(-1)*HR2(0,-1)
				+ 5.0000000000000000e-01*HR1(-1)*HR1(0)*HR1(1)*HR1(1)
				+ 5.8224052646501250e-01*HR1(-1)*HR1(1)
				+ 3.4657359027997265e-01*HR1(-1)*HR1(1)*HR1(1)
				+ HR1( -1)*HR1(1)*HR2(-1,1)
				- HR1( -1)*HR1(1)*HR2(0,-1)
				+ HR1( -1)*HR3(-1,-1,1)
				- HR1( -1)*HR3(-1,1,1)
				+ HR1( -1)*HR3(0,-1,-1)
				+ HR1( -1)*HR3(0,-1,1)
				- HR1(0) *HR1(1)*HR2(-1,1)
				- HR1(0) *HR3(-1,-1,1)
				+ HR1(0) *HR3(-1,1,1)
				- 2.4307035167006157e-01*HR1(1)
				+ 2.9112026323250625e-01*HR1(1)*HR1(1)
				- 5.0000000000000000e-01*HR1(1)*HR1(1)*HR2(0,-1)
				- 6.9314718055994530e-01*HR1(1)*HR2(-1,1)
				- HR1(1) *HR3(-1,-1,1)
				+ HR1(1) *HR3(0,-1,-1)
				+ HR1(1) *HR3(0,-1,1)
				- 6.9314718055994530e-01*HR3(-1,-1,1)
				+ 6.9314718055994530e-01*HR3(-1,1,1)
				- 2.0000000000000000e+00*HR4(-1,-1,-1,1)
				+ HR4( -1,-1,1,1)
				- HR4(0, -1,-1,-1)
				- HR4(0, -1,-1,1)
				- HR4(0, -1,1,-1)
				- HR4(0, -1,1,1);
			HY4(0,0,1,-1) =
				+ 1.7284527823898438e-01
				- 5.0821521280468485e-01*HR1(-1)
				- 5.3134677019160696e-01*HR1(-1)*HR1(-1)
				+ 1.1552453009332421e-01*HR1(-1)*HR1(-1)*HR1(-1)
				- 4.1666666666666666e-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
				- 1.6666666666666666e-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
				- 3.4657359027997265e-01*HR1(-1)*HR1(-1)*HR1(0)
				+ 3.4657359027997265e-01*HR1(-1)*HR1(-1)*HR1(1)
				- 2.5000000000000000e-01*HR1(-1)*HR1(-1)*HR1(1)*HR1(1)
				+ 5.0000000000000000e-01*HR1(-1)*HR1(-1)*HR2(0,-1)
				- 6.9314718055994530e-01*HR1(-1)*HR1(0)*HR1(1)
				- 1.0626935403832139e+00*HR1(-1)*HR1(1)
				+ 3.4657359027997265e-01*HR1(-1)*HR1(1)*HR1(1)
				+ HR1( -1)*HR1(1)*HR2(-1,1)
				+ HR1( -1)*HR1(1)*HR2(0,-1)
				+ 6.9314718055994530e-01*HR1(-1)*HR2(0,-1)
				+ 6.9314718055994530e-01*HR1(-1)*HR2(0,1)
				+ HR1( -1)*HR3(-1,-1,1)
				- HR1( -1)*HR3(-1,1,1)
				- 2.0000000000000000e+00*HR1(-1)*HR3(0,-1,-1)
				- HR1( -1)*HR3(0,-1,1)
				- HR1( -1)*HR3(0,1,-1)
				- 3.4657359027997265e-01*HR1(0)*HR1(1)*HR1(1)
				- 5.0821521280468485e-01*HR1(1)
				- 5.3134677019160696e-01*HR1(1)*HR1(1)
				+ 5.0000000000000000e-01*HR1(1)*HR1(1)*HR2(0,-1)
				- 6.9314718055994530e-01*HR1(1)*HR2(-1,1)
				+ 6.9314718055994530e-01*HR1(1)*HR2(0,-1)
				+ 6.9314718055994530e-01*HR1(1)*HR2(0,1)
				- HR1(1) *HR3(-1,-1,1)
				- 2.0000000000000000e+00*HR1(1)*HR3(0,-1,-1)
				- HR1(1) *HR3(0,-1,1)
				- HR1(1) *HR3(0,1,-1)
				- 6.9314718055994530e-01*HR3(-1,-1,1)
				+ 6.9314718055994530e-01*HR3(-1,1,1)
				- 6.9314718055994530e-01*HR3(0,-1,-1)
				- 6.9314718055994530e-01*HR3(0,-1,1)
				- 6.9314718055994530e-01*HR3(0,1,-1)
				- 6.9314718055994530e-01*HR3(0,1,1)
				- 2.0000000000000000e+00*HR4(-1,-1,-1,1)
				+ HR4( -1,-1,1,1)
				+ 3.0000000000000000e+00*HR4(0,-1,-1,-1)
				+ 2.0000000000000000e+00*HR4(0,-1,-1,1)
				+ 2.0000000000000000e+00*HR4(0,-1,1,-1)
				+ HR4(0, -1,1,1)
				+ 2.0000000000000000e+00*HR4(0,1,-1,-1)
				+ HR4(0,1, -1,1)
				+ HR4(0,1,1, -1);
			HY4(0,-1,0,1) =
				+ 2.0293560632083841e-01
				- 3.8889584616810632e-01*HR1(-1)
				+ 8.2246703342411321e-01*HR1(-1)*HR1(-1)
				+ 1.1552453009332421e-01*HR1(-1)*HR1(-1)*HR1(-1)
				- 4.1666666666666666e-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
				+ 1.6666666666666666e-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
				- 1.6666666666666666e-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
				+ 5.0000000000000000e-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
				+ 3.4657359027997265e-01*HR1(-1)*HR1(-1)*HR1(1)
				+ 5.0000000000000000e-01*HR1(-1)*HR1(-1)*HR2(-1,1)
				- 5.0000000000000000e-01*HR1(-1)*HR1(-1)*HR2(0,-1)
				- 5.0000000000000000e-01*HR1(-1)*HR1(-1)*HR2(0,1)
				- HR1( -1)*HR1(0)*HR2(-1,1)
				+ 1.6449340668482264e+00*HR1(-1)*HR1(1)
				- HR1( -1)*HR1(1)*HR2(-1,1)
				- HR1( -1)*HR1(1)*HR2(0,-1)
				- HR1( -1)*HR1(1)*HR2(0,1)
				- 6.9314718055994530e-01*HR1(-1)*HR2(-1,1)
				- 2.0000000000000000e+00*HR1(-1)*HR3(-1,-1,1)
				+ 2.0000000000000000e+00*HR1(-1)*HR3(-1,1,1)
				+ HR1( -1)*HR3(0,-1,-1)
				+ HR1( -1)*HR3(0,1,-1)
				+ HR1(0) *HR1(1)*HR2(-1,1)
				+ 2.0000000000000000e+00*HR1(0)*HR3(-1,-1,1)
				- 2.0000000000000000e+00*HR1(0)*HR3(-1,1,1)
				- 3.8889584616810632e-01*HR1(1)
				+ 6.9314718055994530e-01*HR1(1)*HR2(-1,1)
				+ 2.0000000000000000e+00*HR1(1)*HR3(-1,-1,1)
				+ HR1(1) *HR3(0,-1,-1)
				+ HR1(1) *HR3(0,1,-1)
				- 1.6449340668482264e+00*HR2(-1,1)
				- 5.0000000000000000e-01*HR2(-1,1)*HR2(-1,1)
				+ HR2( -1,1)*HR2(0,-1)
				+ HR2( -1,1)*HR2(0,1)
				+ 1.3862943611198906e+00*HR3(-1,-1,1)
				- 1.3862943611198906e+00*HR3(-1,1,1)
				+ 4.0000000000000000e+00*HR4(-1,-1,-1,1)
				- 2.0000000000000000e+00*HR4(-1,-1,1,1)
				- HR4(0, -1,-1,-1)
				- HR4(0, -1,-1,1)
				- HR4(0,1, -1,-1)
				- HR4(0,1, -1,1);
			HY4(0,-1,-1,1) =
				+ 3.4159126166513913e-02
				- 9.4753004230127705e-02*HR1(-1)
				+ 2.9112026323250625e-01*HR1(-1)*HR1(-1)
				+ 1.1552453009332421e-01*HR1(-1)*HR1(-1)*HR1(-1)
				- 4.1666666666666666e-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
				+ 1.6666666666666666e-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
				- 1.6666666666666666e-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
				+ 5.0000000000000000e-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
				+ 3.4657359027997265e-01*HR1(-1)*HR1(-1)*HR1(1)
				+ 5.0000000000000000e-01*HR1(-1)*HR1(-1)*HR2(-1,1)
				- 5.0000000000000000e-01*HR1(-1)*HR1(-1)*HR2(0,-1)
				- HR1( -1)*HR1(0)*HR2(-1,1)
				+ 5.8224052646501250e-01*HR1(-1)*HR1(1)
				- HR1( -1)*HR1(1)*HR2(0,-1)
				- 6.9314718055994530e-01*HR1(-1)*HR2(-1,1)
				- HR1( -1)*HR3(-1,-1,1)
				+ HR1( -1)*HR3(0,-1,-1)
				+ HR1(0) *HR3(-1,-1,1)
				- 9.4753004230127705e-02*HR1(1)
				+ HR1(1) *HR3(0,-1,-1)
				- 5.8224052646501250e-01*HR2(-1,1)
				+ HR2( -1,1)*HR2(0,-1)
				+ 6.9314718055994530e-01*HR3(-1,-1,1)
				+ HR4( -1,-1,-1,1)
				- HR4(0, -1,-1,-1)
				- HR4(0, -1,-1,1);
			HY4(0,-1,1,-1) =
				+ 5.4653052738263652e-02
				- 2.1407237086670622e-01*HR1(-1)
				- 5.3134677019160696e-01*HR1(-1)*HR1(-1)
				+ 1.1552453009332421e-01*HR1(-1)*HR1(-1)*HR1(-1)
				- 4.1666666666666666e-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
				- 1.6666666666666666e-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
				- 3.4657359027997265e-01*HR1(-1)*HR1(-1)*HR1(0)
				+ 3.4657359027997265e-01*HR1(-1)*HR1(-1)*HR1(1)
				+ 5.0000000000000000e-01*HR1(-1)*HR1(-1)*HR2(-1,1)
				+ 5.0000000000000000e-01*HR1(-1)*HR1(-1)*HR2(0,-1)
				- 6.9314718055994530e-01*HR1(-1)*HR1(0)*HR1(1)
				- 1.0626935403832139e+00*HR1(-1)*HR1(1)
				+ HR1( -1)*HR1(1)*HR2(0,-1)
				- 6.9314718055994530e-01*HR1(-1)*HR2(-1,1)
				+ 6.9314718055994530e-01*HR1(-1)*HR2(0,-1)
				- HR1( -1)*HR3(-1,-1,1)
				- 2.0000000000000000e+00*HR1(-1)*HR3(0,-1,-1)
				+ 6.9314718055994530e-01*HR1(0)*HR2(-1,1)
				- 2.1407237086670622e-01*HR1(1)
				+ 6.9314718055994530e-01*HR1(1)*HR2(0,-1)
				- 2.0000000000000000e+00*HR1(1)*HR3(0,-1,-1)
				+ 1.0626935403832139e+00*HR2(-1,1)
				- HR2( -1,1)*HR2(0,-1)
				+ 6.9314718055994530e-01*HR3(-1,-1,1)
				- 6.9314718055994530e-01*HR3(0,-1,-1)
				- 6.9314718055994530e-01*HR3(0,-1,1)
				+ HR4( -1,-1,-1,1)
				+ 3.0000000000000000e+00*HR4(0,-1,-1,-1)
				+ 2.0000000000000000e+00*HR4(0,-1,-1,1)
				+ HR4(0, -1,1,-1);
			HY4(0,1,-1,-1) =
				+ 1.1412342741606084e-01
				+ 4.7533770109129867e-01*HR1(-1)
				- 1.2011325347955035e-01*HR1(-1)*HR1(-1)
				+ 1.1552453009332421e-01*HR1(-1)*HR1(-1)*HR1(-1)
				- 4.1666666666666666e-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
				- 1.6666666666666666e-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
				+ 3.4657359027997265e-01*HR1(-1)*HR1(-1)*HR1(1)
				+ 5.0000000000000000e-01*HR1(-1)*HR1(-1)*HR2(-1,1)
				+ 2.4022650695910071e-01*HR1(-1)*HR1(0)
				- 2.4022650695910071e-01*HR1(-1)*HR1(1)
				- 6.9314718055994530e-01*HR1(-1)*HR2(-1,1)
				- 6.9314718055994530e-01*HR1(-1)*HR2(0,-1)
				- HR1( -1)*HR3(-1,-1,1)
				+ HR1( -1)*HR3(0,-1,-1)
				+ 2.4022650695910071e-01*HR1(0)*HR1(1)
				+ 4.7533770109129867e-01*HR1(1)
				- 6.9314718055994530e-01*HR1(1)*HR2(0,-1)
				+ HR1(1) *HR3(0,-1,-1)
				+ 2.4022650695910071e-01*HR2(-1,1)
				- 2.4022650695910071e-01*HR2(0,-1)
				- 2.4022650695910071e-01*HR2(0,1)
				+ 6.9314718055994530e-01*HR3(-1,-1,1)
				+ 1.3862943611198906e+00*HR3(0,-1,-1)
				+ 6.9314718055994530e-01*HR3(0,-1,1)
				+ 6.9314718055994530e-01*HR3(0,1,-1)
				+ HR4( -1,-1,-1,1)
				- 3.0000000000000000e+00*HR4(0,-1,-1,-1)
				- HR4(0, -1,-1,1)
				- HR4(0, -1,1,-1)
				- HR4(0,1, -1,-1);
			HY4(0,-1,1,1) =
				+ 9.3097125991768577e-02
				- 5.3721319360804020e-01*HR1(-1)
				+ 1.2011325347955035e-01*HR1(-1)*HR1(-1)
				- 1.1552453009332421e-01*HR1(-1)*HR1(-1)*HR1(-1)
				+ 4.1666666666666666e-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
				- 1.6666666666666666e-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
				+ 1.6666666666666666e-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
				+ 3.4657359027997265e-01*HR1(-1)*HR1(-1)*HR1(0)
				+ 2.5000000000000000e-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)
				- 5.0000000000000000e-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
				- 3.4657359027997265e-01*HR1(-1)*HR1(-1)*HR1(1)
				- 5.0000000000000000e-01*HR1(-1)*HR1(-1)*HR2(-1,1)
				+ 5.0000000000000000e-01*HR1(-1)*HR1(0)*HR1(0)*HR1(1)
				+ 6.9314718055994530e-01*HR1(-1)*HR1(0)*HR1(1)
				+ HR1( -1)*HR1(0)*HR2(-1,1)
				- HR1( -1)*HR1(0)*HR2(0,-1)
				+ 2.4022650695910071e-01*HR1(-1)*HR1(1)
				+ 6.9314718055994530e-01*HR1(-1)*HR2(-1,1)
				- 6.9314718055994530e-01*HR1(-1)*HR2(0,-1)
				+ HR1( -1)*HR3(-1,-1,1)
				+ HR1( -1)*HR3(0,-1,-1)
				+ HR1( -1)*HR3(0,0,-1)
				- 5.0000000000000000e-01*HR1(0)*HR1(0)*HR2(-1,1)
				- HR1(0) *HR1(1)*HR2(0,-1)
				- 6.9314718055994530e-01*HR1(0)*HR2(-1,1)
				- HR1(0) *HR3(-1,-1,1)
				+ HR1(0) *HR3(0,-1,-1)
				+ HR1(0) *HR3(0,-1,1)
				- 5.3721319360804020e-01*HR1(1)
				- 6.9314718055994530e-01*HR1(1)*HR2(0,-1)
				+ HR1(1) *HR3(0,-1,-1)
				+ HR1(1) *HR3(0,0,-1)
				- 2.4022650695910071e-01*HR2(-1,1)
				- 6.9314718055994530e-01*HR3(-1,-1,1)
				+ 6.9314718055994530e-01*HR3(0,-1,-1)
				+ 6.9314718055994530e-01*HR3(0,-1,1)
				- HR4( -1,-1,-1,1)
				- 2.0000000000000000e+00*HR4(0,-1,-1,-1)
				- HR4(0, -1,-1,1)
				- HR4(0, -1,1,-1)
				- HR4(0,0, -1,-1)
				- HR4(0,0, -1,1);
			HY4(0,1,-1,1) =
				+ 1.9355535381306524e-01
				+ 1.4780047665430420e+00*HR1(-1)
				- 2.9112026323250625e-01*HR1(-1)*HR1(-1)
				- 1.1552453009332421e-01*HR1(-1)*HR1(-1)*HR1(-1)
				+ 4.1666666666666666e-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
				- 1.6666666666666666e-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
				+ 1.6666666666666666e-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
				- 5.0000000000000000e-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(1)
				- 3.4657359027997265e-01*HR1(-1)*HR1(-1)*HR1(1)
				- 5.0000000000000000e-01*HR1(-1)*HR1(-1)*HR2(-1,1)
				+ 5.0000000000000000e-01*HR1(-1)*HR1(-1)*HR2(0,-1)
				+ 5.8224052646501250e-01*HR1(-1)*HR1(0)
				+ HR1( -1)*HR1(0)*HR2(-1,1)
				+ HR1( -1)*HR1(0)*HR2(0,-1)
				- 5.8224052646501250e-01*HR1(-1)*HR1(1)
				+ HR1( -1)*HR1(1)*HR2(0,-1)
				+ 6.9314718055994530e-01*HR1(-1)*HR2(-1,1)
				+ 6.9314718055994530e-01*HR1(-1)*HR2(0,-1)
				+ HR1( -1)*HR3(-1,-1,1)
				- 2.0000000000000000e+00*HR1(-1)*HR3(0,-1,-1)
				- 2.0000000000000000e+00*HR1(-1)*HR3(0,0,-1)
				+ 5.8224052646501250e-01*HR1(0)*HR1(1)
				+ HR1(0) *HR1(1)*HR2(0,-1)
				- HR1(0) *HR3(-1,-1,1)
				- 2.0000000000000000e+00*HR1(0)*HR3(0,-1,-1)
				- HR1(0) *HR3(0,-1,1)
				- HR1(0) *HR3(0,1,-1)
				+ 1.4780047665430420e+00*HR1(1)
				+ 6.9314718055994530e-01*HR1(1)*HR2(0,-1)
				- 2.0000000000000000e+00*HR1(1)*HR3(0,-1,-1)
				- 2.0000000000000000e+00*HR1(1)*HR3(0,0,-1)
				+ 5.8224052646501250e-01*HR2(-1,1)
				- HR2( -1,1)*HR2(0,-1)
				- 5.8224052646501250e-01*HR2(0,-1)
				+ 5.0000000000000000e-01*HR2(0,-1)*HR2(0,-1)
				+ HR2(0, -1)*HR2(0,1)
				- 5.8224052646501250e-01*HR2(0,1)
				- 6.9314718055994530e-01*HR3(-1,-1,1)
				- 1.3862943611198906e+00*HR3(0,-1,-1)
				- 6.9314718055994530e-01*HR3(0,-1,1)
				- 6.9314718055994530e-01*HR3(0,1,-1)
				- HR4( -1,-1,-1,1)
				+ 4.0000000000000000e+00*HR4(0,-1,-1,-1)
				+ 2.0000000000000000e+00*HR4(0,-1,-1,1)
				- HR4(0, -1,0,1)
				+ HR4(0, -1,1,-1)
				+ 2.0000000000000000e+00*HR4(0,0,-1,-1)
				+ HR4(0,1, -1,-1);
			HY4(0,1,1,-1) =
				+ 4.3369237704895519e-01
				- 1.1073038989294665e+00*HR1(-1)
				+ 5.3134677019160696e-01*HR1(-1)*HR1(-1)
				- 1.1552453009332421e-01*HR1(-1)*HR1(-1)*HR1(-1)
				+ 4.1666666666666666e-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
				+ 1.6666666666666666e-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(1)
				+ 3.4657359027997265e-01*HR1(-1)*HR1(-1)*HR1(0)
				- 3.4657359027997265e-01*HR1(-1)*HR1(-1)*HR1(1)
				- 5.0000000000000000e-01*HR1(-1)*HR1(-1)*HR2(-1,1)
				- 5.0000000000000000e-01*HR1(-1)*HR1(-1)*HR2(0,-1)
				- 1.0626935403832139e+00*HR1(-1)*HR1(0)
				- 3.4657359027997265e-01*HR1(-1)*HR1(0)*HR1(0)
				+ 6.9314718055994530e-01*HR1(-1)*HR1(0)*HR1(1)
				+ 1.0626935403832139e+00*HR1(-1)*HR1(1)
				- HR1( -1)*HR1(1)*HR2(0,-1)
				+ 6.9314718055994530e-01*HR1(-1)*HR2(-1,1)
				+ HR1( -1)*HR3(-1,-1,1)
				+ HR1( -1)*HR3(0,-1,-1)
				+ HR1( -1)*HR3(0,0,-1)
				- 3.4657359027997265e-01*HR1(0)*HR1(0)*HR1(1)
				- 1.0626935403832139e+00*HR1(0)*HR1(1)
				- 6.9314718055994530e-01*HR1(0)*HR2(-1,1)
				+ 6.9314718055994530e-01*HR1(0)*HR2(0,-1)
				+ 6.9314718055994530e-01*HR1(0)*HR2(0,1)
				- 1.1073038989294665e+00*HR1(1)
				+ HR1(1) *HR3(0,-1,-1)
				+ HR1(1) *HR3(0,0,-1)
				- 1.0626935403832139e+00*HR2(-1,1)
				+ HR2( -1,1)*HR2(0,-1)
				+ 1.0626935403832139e+00*HR2(0,-1)
				- 5.0000000000000000e-01*HR2(0,-1)*HR2(0,-1)
				- HR2(0, -1)*HR2(0,1)
				+ 1.0626935403832139e+00*HR2(0,1)
				- 6.9314718055994530e-01*HR3(-1,-1,1)
				- 6.9314718055994530e-01*HR3(0,-1,-1)
				- 6.9314718055994530e-01*HR3(0,0,-1)
				- 6.9314718055994530e-01*HR3(0,0,1)
				- 6.9314718055994530e-01*HR3(0,1,-1)
				- HR4( -1,-1,-1,1)
				- HR4(0, -1,-1,1)
				+ HR4(0, -1,0,1)
				+ HR4(0,0, -1,1)
				+ HR4(0,0,1, -1)
				+ HR4(0,1, -1,-1);
			HY4(-1,-1,-1,1) =
				+ 1.4134237214990008e-02
				- 9.4753004230127705e-02*HR1(-1)
				+ 2.9112026323250625e-01*HR1(-1)*HR1(-1)
				+ 1.1552453009332421e-01*HR1(-1)*HR1(-1)*HR1(-1)
				- 4.1666666666666666e-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
				+ 1.6666666666666666e-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
				- 5.0000000000000000e-01*HR1(-1)*HR1(-1)*HR2(0,-1)
				+ HR1( -1)*HR3(0,-1,-1)
				- HR4(0, -1,-1,-1);
			HY4(-1,-1,1,1) =
				+ 4.0758239159309251e-02
				- 5.3721319360804020e-01*HR1(-1)
				+ 1.2011325347955035e-01*HR1(-1)*HR1(-1)
				- 1.1552453009332421e-01*HR1(-1)*HR1(-1)*HR1(-1)
				+ 4.1666666666666666e-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
				- 1.6666666666666666e-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
				+ 3.4657359027997265e-01*HR1(-1)*HR1(-1)*HR1(0)
				+ 2.5000000000000000e-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)
				- HR1( -1)*HR1(0)*HR2(0,-1)
				- 6.9314718055994530e-01*HR1(-1)*HR2(0,-1)
				+ HR1( -1)*HR3(0,-1,-1)
				+ HR1( -1)*HR3(0,0,-1)
				+ HR1(0) *HR3(0,-1,-1)
				+ 6.9314718055994530e-01*HR3(0,-1,-1)
				- 2.0000000000000000e+00*HR4(0,-1,-1,-1)
				- HR4(0,0, -1,-1);
			HY4(-1,1,1,1) =
				+ 5.1747906167389938e-01
				+ 5.5504108664821579e-02*HR1(-1)
				- 1.2011325347955035e-01*HR1(-1)*HR1(-1)
				+ 1.1552453009332421e-01*HR1(-1)*HR1(-1)*HR1(-1)
				- 4.1666666666666666e-02*HR1(-1)*HR1(-1)*HR1(-1)*HR1(-1)
				+ 1.6666666666666666e-01*HR1(-1)*HR1(-1)*HR1(-1)*HR1(0)
				- 3.4657359027997265e-01*HR1(-1)*HR1(-1)*HR1(0)
				- 2.5000000000000000e-01*HR1(-1)*HR1(-1)*HR1(0)*HR1(0)
				+ 2.4022650695910071e-01*HR1(-1)*HR1(0)
				+ 3.4657359027997265e-01*HR1(-1)*HR1(0)*HR1(0)
				+ 1.6666666666666666e-01*HR1(-1)*HR1(0)*HR1(0)*HR1(0)
				- 5.0000000000000000e-01*HR1(0)*HR1(0)*HR2(0,-1)
				- 6.9314718055994530e-01*HR1(0)*HR2(0,-1)
				+ HR1(0) *HR3(0,-1,-1)
				+ HR1(0) *HR3(0,0,-1)
				- 2.4022650695910071e-01*HR2(0,-1)
				+ 6.9314718055994530e-01*HR3(0,-1,-1)
				+ 6.9314718055994530e-01*HR3(0,0,-1)
				- HR4(0, -1,-1,-1)
				- HR4(0,0, -1,-1)
				- HR4(0,0,0, -1);
			if (r<0.)
			{
				HY4(0,-1,1,1) = HY4(0,-1,1,1)
					- 2.4674011002723396e+00*HR1(-1)*HR1(-1)
					- 4.9348022005446793e+00*HR1(-1)*HR1(1)
					+ 4.9348022005446793e+00*HR2(-1,1);
				HY4(0,1,1,-1) = HY4(0,1,1,-1)
					+ 3.4205442319285582e+00*HR1(-1)
					+ 3.4205442319285582e+00*HR1(1);
				HY4(-1,-1,1,1) = HY4(-1,-1,1,1)
					- 2.4674011002723396e+00*HR1(-1)*HR1(-1);
				HY4(-1,1,1,1) = HY4(-1,1,1,1)
					- 3.4205442319285582e+00*HR1(-1)
					+ 2.4674011002723396e+00*HR1(-1)*HR1(-1)
					- 4.9348022005446793e+00*HR1(-1)*HR1(0)
					+ 4.9348022005446793e+00*HR2(0,-1);
				Hi4(0,0,-1,1) =
					- 1.6666666666666666e-01*HR1(-1)*HR1(-1)*HR1(-1)
					- 5.0000000000000000e-01*HR1(-1)*HR1(-1)*HR1(1)
					- 5.0000000000000000e-01*HR1(-1)*HR1(1)*HR1(1)
					+ HR1(1) *HR2(-1,1)
					+ HR3( -1,-1,1)
					- HR3( -1,1,1);
				Hi4(0,0,1,-1) =
					+ 3.4657359027997265e-01*HR1(-1)*HR1(-1)
					+ 6.9314718055994530e-01*HR1(-1)*HR1(1)
					+ 3.4657359027997265e-01*HR1(1)*HR1(1);
				Hi4(0,-1,0,1) =
					- 1.6666666666666666e-01*HR1(-1)*HR1(-1)*HR1(-1)
					- 5.0000000000000000e-01*HR1(-1)*HR1(-1)*HR1(1)
					+ HR1( -1)*HR2(-1,1)
					- HR1(1) *HR2(-1,1)
					- 2.0000000000000000e+00*HR3(-1,-1,1)
					+ 2.0000000000000000e+00*HR3(-1,1,1);
				Hi4(0,-1,-1,1) =
					- 1.6666666666666666e-01*HR1(-1)*HR1(-1)*HR1(-1)
					- 5.0000000000000000e-01*HR1(-1)*HR1(-1)*HR1(1)
					+ HR1( -1)*HR2(-1,1)
					- HR3( -1,-1,1);
				Hi4(0,-1,1,-1) =
					+ 3.4657359027997265e-01*HR1(-1)*HR1(-1)
					+ 6.9314718055994530e-01*HR1(-1)*HR1(1)
					- 6.9314718055994530e-01*HR2(-1,1);
				Hi4(0,1,-1,-1) =
					- 2.4022650695910071e-01*HR1(-1)
					- 2.4022650695910071e-01*HR1(1);
				Hi4(0,-1,1,1) =
					- 3.4657359027997265e-01*HR1(-1)*HR1(-1)
					+ 1.6666666666666666e-01*HR1(-1)*HR1(-1)*HR1(-1)
					- 5.0000000000000000e-01*HR1(-1)*HR1(-1)*HR1(0)
					+ 5.0000000000000000e-01*HR1(-1)*HR1(-1)*HR1(1)
					- HR1( -1)*HR1(0)*HR1(1)
					- 6.9314718055994530e-01*HR1(-1)*HR1(1)
					- HR1( -1)*HR2(-1,1)
					+ HR1( -1)*HR2(0,-1)
					+ HR1(0) *HR2(-1,1)
					+ HR1(1) *HR2(0,-1)
					+ 6.9314718055994530e-01*HR2(-1,1)
					+ HR3( -1,-1,1)
					- HR3(0, -1,-1)
					- HR3(0, -1,1);
				Hi4(0,1,-1,1) =
					- 5.8224052646501250e-01*HR1(-1)
					+ 1.6666666666666666e-01*HR1(-1)*HR1(-1)*HR1(-1)
					+ 5.0000000000000000e-01*HR1(-1)*HR1(-1)*HR1(1)
					- HR1( -1)*HR2(-1,1)
					- HR1( -1)*HR2(0,-1)
					- 5.8224052646501250e-01*HR1(1)
					- HR1(1) *HR2(0,-1)
					+ HR3( -1,-1,1)
					+ 2.0000000000000000e+00*HR3(0,-1,-1)
					+ HR3(0, -1,1)
					+ HR3(0,1, -1);
				Hi4(0,1,1,-1) =
					+ 1.0626935403832139e+00*HR1(-1)
					- 3.4657359027997265e-01*HR1(-1)*HR1(-1)
					+ 6.9314718055994530e-01*HR1(-1)*HR1(0)
					- 6.9314718055994530e-01*HR1(-1)*HR1(1)
					+ 6.9314718055994530e-01*HR1(0)*HR1(1)
					+ 1.0626935403832139e+00*HR1(1)
					+ 6.9314718055994530e-01*HR2(-1,1)
					- 6.9314718055994530e-01*HR2(0,-1)
					- 6.9314718055994530e-01*HR2(0,1);
				Hi4(-1,-1,-1,1) =
					- 1.6666666666666666e-01*HR1(-1)*HR1(-1)*HR1(-1);
				Hi4(-1,-1,1,1) =
					- 3.4657359027997265e-01*HR1(-1)*HR1(-1)
					+ 1.6666666666666666e-01*HR1(-1)*HR1(-1)*HR1(-1)
					- 5.0000000000000000e-01*HR1(-1)*HR1(-1)*HR1(0)
					+ HR1( -1)*HR2(0,-1)
					- HR3(0, -1,-1);
				Hi4(-1,1,1,1) =
					+ 1.4047075598891257e+00*HR1(-1)
					+ 3.4657359027997265e-01*HR1(-1)*HR1(-1)
					- 1.6666666666666666e-01*HR1(-1)*HR1(-1)*HR1(-1)
					+ 5.0000000000000000e-01*HR1(-1)*HR1(-1)*HR1(0)
					- 6.9314718055994530e-01*HR1(-1)*HR1(0)
					- 5.0000000000000000e-01*HR1(-1)*HR1(0)*HR1(0)
					+ HR1(0) *HR2(0,-1)
					+ 6.9314718055994530e-01*HR2(0,-1)
					- HR3(0, -1,-1)
					- HR3(0,0, -1);
			}
		} // nw > 3 endif
	} // (n1,n2) = (-1,1) -- completion endif
	return;
}


/**
 *
 * @brief evaluates the HPL for y > r2p1
 * fillirr1dhplatinf is called by eval1dhplatinf after calling
 * fillirr1dhplat0 with argument r=1/y
 * it is guaranteed that nw is in the range 2:4, and that (n1,n2)
 * take one of the pairs of values (0,1), (-1,0) or (-1,1)
 */
void hpl::details::fillirr1dhplatinf(double x, int nw,
		FA1dD11& HX1, FA2dD11& HX2, FA3dD11& HX3, FA4dD11& HX4,
		FA1dD11& HY1, FA2dD11& HY2, FA3dD11& HY3, FA4dD11& HY4,
		FA1dD11& Hi1, FA2dD11& Hi2, FA3dD11& Hi3, FA4dD11& Hi4,
		int n1, int n2)
{
	// (n1,n2) = (0,1) or (-1,1)
	if ( ( (n1==0) && (n2==1) ) || ( (n1==-1) && (n2==1) ) )
	{
		HY2(0,1) =
			+ 3.2898681336964528e+00
			- 5.0000000000000000e-01*HX1(0)*HX1(0)
			- HX2(0,1);
		Hi2(0,1) = - HX1(0);
		if ( nw>2 )
		{
			HY3(0,0,1) =
				- 3.2898681336964528e+00*HX1(0)
				+ 1.6666666666666666e-01*HX1(0)*HX1(0)*HX1(0)
				+ HX3(0,0,1);
			HY3(0,1,1) =
				+ 1.2020569031595942e+00
				+ 4.9348022005446793e+00*HX1(0)
				- 1.6666666666666666e-01*HX1(0)*HX1(0)*HX1(0)
				- HX1(0) *HX2(0,1)
				+ HX3(0,0,1)
				- HX3(0,1,1);
			Hi3(0,0,1) =
				+ 5.000000000000000e-01*HX1(0)*HX1(0);
			Hi3(0,1,1) =
				+ 1.6449340668482264e+00
				- 5.0000000000000000e-01*HX1(0)*HX1(0)
				- HX2(0,1);
		}
		if ( nw>3 )
		{
			HY4(0,0,0,1) =
				+ 2.1646464674222763e+00
				+ 1.6449340668482264e+00*HX1(0)*HX1(0)
				- 4.1666666666666666e-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)
				- HX4(0,0,0,1);
			HY4(0,0,1,1) =
				+ 2.1646464674222763e+00
				- 1.2020569031595942e+00*HX1(0)
				- 2.4674011002723396e+00*HX1(0)*HX1(0)
				+ 4.1666666666666666e-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)
				+ HX1(0) *HX3(0,0,1)
				- 2.0000000000000000e+00*HX4(0,0,0,1)
				+ HX4(0,0,1,1);
			HY4(0,1,1,1) =
				- 5.1410353601279064e+00
				+ 2.4674011002723396e+00*HX1(0)*HX1(0)
				- 4.1666666666666666e-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)
				- 5.0000000000000000e-01*HX1(0)*HX1(0)*HX2(0,1)
				+ HX1(0) *HX3(0,0,1)
				- HX1(0) *HX3(0,1,1)
				+ 4.9348022005446793e+00*HX2(0,1)
				- HX4(0,0,0,1)
				+ HX4(0,0,1,1)
				- HX4(0,1,1,1);
			Hi4(0,0,0,1) =
				- 1.6666666666666666e-01*HX1(0)*HX1(0)*HX1(0);
			Hi4(0,0,1,1) =
				- 1.2020569031595942e+00
				- 1.6449340668482264e+00*HX1(0)
				+ 1.6666666666666666e-01*HX1(0)*HX1(0)*HX1(0)
				+ HX3(0,0,1);
			Hi4(0,1,1,1) =
				+ 1.6449340668482264e+00*HX1(0)
				- 1.6666666666666666e-01*HX1(0)*HX1(0)*HX1(0)
				- HX1(0) *HX2(0,1)
				+ HX3(0,0,1)
				- HX3(0,1,1);
		} // nw > 3 endif
	} // (n1,n2) = (0,1) or (-1,1) endif

	//************
	// (n1,n2) = (-1,0) or (-1,1)
	if ( ( (n1==-1) && (n2==0) ) || ( (n1==-1) && (n2==1) ) )
	{
		HY2(0,-1) =
			+ 1.6449340668482264e+00
			+ 5.0000000000000000e-01*HX1(0)*HX1(0)
			- HX2(0, -1);
		if ( nw>2 ){
			HY3(0,0,-1) =
				- 1.6449340668482264e+00*HX1(0)
				- 1.6666666666666666e-01*HX1(0)*HX1(0)*HX1(0)
				+ HX3(0,0, -1);
			HY3(0,-1,-1) =
				+ 1.2020569031595942e+00
				- 1.6666666666666666e-01*HX1(0)*HX1(0)*HX1(0)
				+ HX1(0) *HX2(0,-1)
				- HX3(0, -1,-1)
				- HX3(0,0, -1);
		}
		if ( nw>3 )
		{
			HY4(0,0,0,-1) =
				+ 1.8940656589944918e+00
				+ 8.2246703342411321e-01*HX1(0)*HX1(0)
				+ 4.1666666666666666e-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)
				- HX4(0,0,0, -1);
			HY4(0,0,-1,-1) =
				- 1.8940656589944918e+00
				- 1.2020569031595942e+00*HX1(0)
				+ 4.1666666666666666e-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)
				- HX1(0) *HX3(0,0,-1)
				+ HX4(0,0, -1,-1)
				+ 2.0000000000000000e+00*HX4(0,0,0,-1);
			HY4(0,-1,-1,-1) =
				+ 1.0823232337111381e+00
				+ 4.1666666666666666e-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)
				- 5.0000000000000000e-01*HX1(0)*HX1(0)*HX2(0,-1)
				+ HX1(0) *HX3(0,-1,-1)
				+ HX1(0) *HX3(0,0,-1)
				- HX4(0, -1,-1,-1)
				- HX4(0,0, -1,-1)
				- HX4(0,0,0, -1);
		} // nw > 3 endif
	} // (n1,n2) = (-1,0) or (-1,1) endif
	// (n1,n2) = (-1,1) -- completion
	if ( (n1==-1) && (n2==1) )
	{
		HY2(-1,1) =
			+ 2.4674011002723396e+00
			+ HX1( -1)*HX1(0)
			- 5.0000000000000000e-01*HX1(0)*HX1(0)
			+ HX2( -1,1)
			- HX2(0, -1)
			- HX2(0,1);
		Hi2(-1,1) =
			- 6.9314718055994530e-01
			+ HX1( -1)
			- HX1(0);
		if ( nw>2 )
		{
			HY3(0,-1,1) =
				- 2.5190015545588625e+00
				- 2.4674011002723396e+00*HX1(0)
				+ 1.6666666666666666e-01*HX1(0)*HX1(0)*HX1(0)
				- HX1(0) *HX2(0,-1)
				- HX3(0, -1,1)
				+ 2.0000000000000000e+00*HX3(0,0,-1)
				+ HX3(0,0,1);
			HY3(0,1,-1) =
				+ 4.3220869092982539e+00
				+ 2.4674011002723396e+00*HX1(0)
				+ 1.6666666666666666e-01*HX1(0)*HX1(0)*HX1(0)
				+ HX1(0) *HX2(0,1)
				- HX3(0,0, -1)
				- 2.0000000000000000e+00*HX3(0,0,1)
				- HX3(0,1, -1);
			HY3(-1,-1,1) =
				- 2.7620719062289241e+00
				+ 2.4674011002723396e+00*HX1(-1)
				+ 5.0000000000000000e-01*HX1(-1)*HX1(-1)*HX1(0)
				- 5.0000000000000000e-01*HX1(-1)*HX1(0)*HX1(0)
				- HX1( -1)*HX2(0,-1)
				- HX1( -1)*HX2(0,1)
				- 2.4674011002723396e+00*HX1(0)
				+ 1.6666666666666666e-01*HX1(0)*HX1(0)*HX1(0)
				+ HX3( -1,-1,1)
				+ HX3(0, -1,-1)
				+ HX3(0,0, -1)
				+ HX3(0,0,1)
				+ HX3(0,1, -1);
			HY3(-1,1,1) =
				+ 2.7620719062289241e+00
				- 4.9348022005446793e+00*HX1(-1)
				+ 5.0000000000000000e-01*HX1(-1)*HX1(0)*HX1(0)
				+ 4.9348022005446793e+00*HX1(0)
				- 1.6666666666666666e-01*HX1(0)*HX1(0)*HX1(0)
				+ HX1(0) *HX2(-1,1)
				- HX1(0) *HX2(0,-1)
				- HX1(0) *HX2(0,1)
				+ HX3( -1,1,1)
				- HX3(0, -1,1)
				+ HX3(0,0, -1)
				+ HX3(0,0,1)
				- HX3(0,1,1);
			Hi3(0,-1,1) =
				+ 8.2246703342411321e-01
				+ 6.9314718055994530e-01*HX1(0)
				+ 5.0000000000000000e-01*HX1(0)*HX1(0)
				- HX2(0, -1);
			Hi3(0,1,-1) =
				- 6.9314718055994530e-01*HX1(0);
			Hi3(-1,-1,1) =
				+ 2.4022650695910071e-01
				- 6.9314718055994530e-01*HX1(-1)
				+ 5.0000000000000000e-01*HX1(-1)*HX1(-1)
				- HX1( -1)*HX1(0)
				+ 6.9314718055994530e-01*HX1(0)
				+ 5.0000000000000000e-01*HX1(0)*HX1(0);
			Hi3(-1,1,1) =
				+ 1.8851605738073271e+00
				+ HX1( -1)*HX1(0)
				- 5.0000000000000000e-01*HX1(0)*HX1(0)
				+ HX2( -1,1)
				- HX2(0, -1)
				- HX2(0,1);
		}
		if ( nw>3 )
		{
			HY4(0,0,-1,1) =
				+ 3.9234217222028759e+00
				+ 2.5190015545588625e+00*HX1(0)
				+ 1.2337005501361698e+00*HX1(0)*HX1(0)
				- 4.1666666666666666e-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)
				+ HX1(0) *HX3(0,0,-1)
				+ HX4(0,0, -1,1)
				- 3.0000000000000000e+00*HX4(0,0,0,-1)
				- HX4(0,0,0,1);
			HY4(0,0,1,-1) =
				- 4.1940025306306604e+00
				- 4.3220869092982539e+00*HX1(0)
				- 1.2337005501361698e+00*HX1(0)*HX1(0)
				- 4.1666666666666666e-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)
				- HX1(0) *HX3(0,0,1)
				+ HX4(0,0,0, -1)
				+ 3.0000000000000000e+00*HX4(0,0,0,1)
				+ HX4(0,0,1, -1);
			HY4(0,-1,0,1) =
				+ 9.4703282949724591e-01
				+ 1.8030853547393914e+00*HX1(0)
				+ 1.6449340668482264e+00*HX1(0)*HX1(0)
				- 4.1666666666666666e-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)
				+ 5.0000000000000000e-01*HX1(0)*HX1(0)*HX2(0,-1)
				- 2.0000000000000000e+00*HX1(0)*HX3(0,0,-1)
				- 3.2898681336964528e+00*HX2(0,-1)
				+ HX4(0, -1,0,1)
				+ 3.0000000000000000e+00*HX4(0,0,0,-1)
				- HX4(0,0,0,1);
			HY4(0,-1,-1,1) =
				+ 2.5209599327464717e+00
				+ 2.7620719062289241e+00*HX1(0)
				+ 1.2337005501361698e+00*HX1(0)*HX1(0)
				- 4.1666666666666666e-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)
				+ 5.0000000000000000e-01*HX1(0)*HX1(0)*HX2(0,-1)
				- HX1(0) *HX3(0,-1,-1)
				- HX1(0) *HX3(0,0,-1)
				- 2.4674011002723396e+00*HX2(0,-1)
				+ 5.0000000000000000e-01*HX2(0,-1)*HX2(0,-1)
				- HX4(0, -1,-1,1)
				+ HX4(0, -1,0,1)
				+ HX4(0,0, -1,1)
				- HX4(0,0,0,1);
			HY4(0,-1,1,-1) =
				- 8.5266539820739622e+00
				- 5.5241438124578482e+00*HX1(0)
				- 1.2337005501361698e+00*HX1(0)*HX1(0)
				- 4.1666666666666666e-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)
				+ 5.0000000000000000e-01*HX1(0)*HX1(0)*HX2(0,-1)
				+ HX1(0) *HX3(0,-1,1)
				- 2.0000000000000000e+00*HX1(0)*HX3(0,0,-1)
				- HX1(0) *HX3(0,0,1)
				+ 2.4674011002723396e+00*HX2(0,-1)
				- 5.0000000000000000e-01*HX2(0,-1)*HX2(0,-1)
				- HX4(0, -1,0,1)
				- HX4(0, -1,1,-1)
				+ 2.0000000000000000e+00*HX4(0,0,-1,-1)
				- 2.0000000000000000e+00*HX4(0,0,-1,1)
				+ 4.0000000000000000e+00*HX4(0,0,0,-1)
				+ 3.0000000000000000e+00*HX4(0,0,0,1)
				+ HX4(0,0,1, -1);
			HY4(0,1,-1,-1) =
				+ 5.8027584430066521e+00
				+ 2.7620719062289241e+00*HX1(0)
				- 4.1666666666666666e-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)
				- 5.0000000000000000e-01*HX1(0)*HX1(0)*HX2(0,1)
				+ HX1(0) *HX3(0,0,-1)
				+ 2.0000000000000000e+00*HX1(0)*HX3(0,0,1)
				+ HX1(0) *HX3(0,1,-1)
				- HX4(0,0, -1,-1)
				- 2.0000000000000000e+00*HX4(0,0,0,-1)
				- 3.0000000000000000e+00*HX4(0,0,0,1)
				- 2.0000000000000000e+00*HX4(0,0,1,-1)
				- HX4(0,1, -1,-1);
			HY4(0,-1,1,1) =
				+ 6.2689427375197987e-01
				- 2.7620719062289241e+00*HX1(0)
				- 2.4674011002723396e+00*HX1(0)*HX1(0)
				+ 4.1666666666666666e-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)
				- 5.0000000000000000e-01*HX1(0)*HX1(0)*HX2(0,-1)
				- HX1(0) *HX3(0,-1,1)
				+ 2.0000000000000000e+00*HX1(0)*HX3(0,0,-1)
				+ HX1(0) *HX3(0,0,1)
				+ 4.9348022005446793e+00*HX2(0,-1)
				- HX4(0, -1,1,1)
				+ 2.0000000000000000e+00*HX4(0,0,-1,1)
				- 3.0000000000000000e+00*HX4(0,0,0,-1)
				- 2.0000000000000000e+00*HX4(0,0,0,1)
				+ HX4(0,0,1,1);
			HY4(0,1,-1,1) =
				- 4.3326514514433017e+00
				- 1.3169446513992682e+00*HX1(0)
				- 1.2337005501361698e+00*HX1(0)*HX1(0)
				+ 4.1666666666666666e-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)
				+ 5.0000000000000000e-01*HX1(0)*HX1(0)*HX2(0,1)
				- HX1(0) *HX3(0,0,-1)
				- 2.0000000000000000e+00*HX1(0)*HX3(0,0,1)
				- HX1(0) *HX3(0,1,-1)
				+ HX2(0, -1)*HX2(0,1)
				- 2.4674011002723396e+00*HX2(0,1)
				+ 5.0000000000000000e-01*HX2(0,1)*HX2(0,1)
				- HX4(0, -1,0,1)
				- 3.0000000000000000e+00*HX4(0,0,-1,1)
				+ 3.0000000000000000e+00*HX4(0,0,0,-1)
				+ 4.0000000000000000e+00*HX4(0,0,0,1)
				- 2.0000000000000000e+00*HX4(0,0,1,1)
				- HX4(0,1, -1,1);
			HY4(0,1,1,-1) =
				- 1.5001934240460787e-01
				+ 4.0790165576281924e+00*HX1(0)
				+ 1.2337005501361698e+00*HX1(0)*HX1(0)
				+ 4.1666666666666666e-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)
				+ 5.0000000000000000e-01*HX1(0)*HX1(0)*HX2(0,1)
				- HX1(0) *HX3(0,0,1)
				+ HX1(0) *HX3(0,1,1)
				- HX2(0, -1)*HX2(0,1)
				+ 2.4674011002723396e+00*HX2(0,1)
				- 5.0000000000000000e-01*HX2(0,1)*HX2(0,1)
				+ HX4(0, -1,0,1)
				+ 2.0000000000000000e+00*HX4(0,0,-1,1)
				- HX4(0,0,0, -1)
				+ HX4(0,0,1, -1)
				- HX4(0,1,1, -1);
			HY4(-1,-1,-1,1) =
				+ 2.4278628067547031e+00
				- 2.7620719062289241e+00*HX1(-1)
				+ 1.2337005501361698e+00*HX1(-1)*HX1(-1)
				+ 1.6666666666666666e-01*HX1(-1)*HX1(-1)*HX1(-1)*HX1(0)
				- 2.5000000000000000e-01*HX1(-1)*HX1(-1)*HX1(0)*HX1(0)
				- 5.0000000000000000e-01*HX1(-1)*HX1(-1)*HX2(0,-1)
				- 5.0000000000000000e-01*HX1(-1)*HX1(-1)*HX2(0,1)
				- 2.4674011002723396e+00*HX1(-1)*HX1(0)
				+ 1.6666666666666666e-01*HX1(-1)*HX1(0)*HX1(0)*HX1(0)
				+ HX1( -1)*HX3(0,-1,-1)
				+ HX1( -1)*HX3(0,0,-1)
				+ HX1( -1)*HX3(0,0,1)
				+ HX1( -1)*HX3(0,1,-1)
				+ 2.7620719062289241e+00*HX1(0)
				+ 1.2337005501361698e+00*HX1(0)*HX1(0)
				- 4.1666666666666666e-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)
				+ HX4( -1,-1,-1,1)
				- HX4(0, -1,-1,-1)
				- HX4(0,0, -1,-1)
				- HX4(0,0,0, -1)
				- HX4(0,0,0,1)
				- HX4(0,0,1, -1)
				- HX4(0,1, -1,-1);
			HY4(-1,-1,1,1) =
				+ 2.0293560632083841e+00
				+ 2.7620719062289241e+00*HX1(-1)
				- 2.4674011002723396e+00*HX1(-1)*HX1(-1)
				+ 2.5000000000000000e-01*HX1(-1)*HX1(-1)*HX1(0)*HX1(0)
				+ 4.9348022005446793e+00*HX1(-1)*HX1(0)
				- 1.6666666666666666e-01*HX1(-1)*HX1(0)*HX1(0)*HX1(0)
				- HX1( -1)*HX1(0)*HX2(0,-1)
				- HX1( -1)*HX1(0)*HX2(0,1)
				- HX1( -1)*HX3(0,-1,1)
				+ HX1( -1)*HX3(0,0,-1)
				+ HX1( -1)*HX3(0,0,1)
				- HX1( -1)*HX3(0,1,1)
				- 2.7620719062289241e+00*HX1(0)
				- 2.4674011002723396e+00*HX1(0)*HX1(0)
				+ 4.1666666666666666e-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)
				+ HX1(0) *HX3(-1,-1,1)
				+ HX1(0) *HX3(0,-1,-1)
				+ HX1(0) *HX3(0,0,-1)
				+ HX1(0) *HX3(0,0,1)
				+ HX1(0) *HX3(0,1,-1)
				+ HX4( -1,-1,1,1)
				+ HX4(0, -1,-1,1)
				+ HX4(0, -1,1,-1)
				- HX4(0,0, -1,-1)
				+ HX4(0,0, -1,1)
				- 2.0000000000000000e+00*HX4(0,0,0,-1)
				- 2.0000000000000000e+00*HX4(0,0,0,1)
				- HX4(0,0,1, -1)
				+ HX4(0,0,1,1)
				+ HX4(0,1, -1,1)
				+ HX4(0,1,1, -1);
			HY4(-1,1,1,1) =
				- 6.4865749331714713e+00
				- 4.9348022005446793e+00*HX1(-1)*HX1(0)
				+ 1.6666666666666666e-01*HX1(-1)*HX1(0)*HX1(0)*HX1(0)
				+ 2.4674011002723396e+00*HX1(0)*HX1(0)
				- 4.1666666666666666e-02*HX1(0)*HX1(0)*HX1(0)*HX1(0)
				+ 5.0000000000000000e-01*HX1(0)*HX1(0)*HX2(-1,1)
				- 5.0000000000000000e-01*HX1(0)*HX1(0)*HX2(0,-1)
				- 5.0000000000000000e-01*HX1(0)*HX1(0)*HX2(0,1)
				+ HX1(0) *HX3(-1,1,1)
				- HX1(0) *HX3(0,-1,1)
				+ HX1(0) *HX3(0,0,-1)
				+ HX1(0) *HX3(0,0,1)
				- HX1(0) *HX3(0,1,1)
				- 4.9348022005446793e+00*HX2(-1,1)
				+ 4.9348022005446793e+00*HX2(0,-1)
				+ 4.9348022005446793e+00*HX2(0,1)
				+ HX4( -1,1,1,1)
				- HX4(0, -1,1,1)
				+ HX4(0,0, -1,1)
				- HX4(0,0,0, -1)
				- HX4(0,0,0,1)
				+ HX4(0,0,1,1)
				- HX4(0,1,1,1);
			Hi4(0,0,-1,1) =
				- 9.0154267736969571e-01
				- 8.2246703342411321e-01*HX1(0)
				- 3.4657359027997265e-01*HX1(0)*HX1(0)
				- 1.6666666666666666e-01*HX1(0)*HX1(0)*HX1(0)
				+ HX3(0,0, -1);
			Hi4(0,0,1,-1) =
				+ 3.4657359027997265e-01*HX1(0)*HX1(0);
			Hi4(0,-1,0,1) =
				+ 1.8030853547393914e+00
				+ 8.2246703342411321e-01*HX1(0)
				- 1.6666666666666666e-01*HX1(0)*HX1(0)*HX1(0)
				+ HX1(0) *HX2(0,-1)
				- 2.0000000000000000e+00*HX3(0,0,-1);
			Hi4(0,-1,-1,1) =
				+ 4.8170908494321862e-01
				- 2.4022650695910071e-01*HX1(0)
				- 3.4657359027997265e-01*HX1(0)*HX1(0)
				- 1.6666666666666666e-01*HX1(0)*HX1(0)*HX1(0)
				+ HX1(0) *HX2(0,-1)
				+ 6.9314718055994530e-01*HX2(0,-1)
				- HX3(0, -1,-1)
				- HX3(0,0, -1);
			Hi4(0,-1,1,-1) =
				+ 5.7009070532142637e-01
				+ 4.8045301391820142e-01*HX1(0)
				+ 3.4657359027997265e-01*HX1(0)*HX1(0)
				- 6.9314718055994530e-01*HX2(0,-1);
			Hi4(0,1,-1,-1) =
				- 2.4022650695910071e-01*HX1(0);
			Hi4(0,-1,1,1) =
				- 2.7620719062289241e+00
				- 1.8851605738073271e+00*HX1(0)
				+ 1.6666666666666666e-01*HX1(0)*HX1(0)*HX1(0)
				- HX1(0) *HX2(0,-1)
				- HX3(0, -1,1)
				+ 2.0000000000000000e+00*HX3(0,0,-1)
				+ HX3(0,0,1);
			Hi4(0,1,-1,1) =
				+ 2.6736902858507163e+00
				+ 1.3029200473423146e+00*HX1(0)
				+ 3.4657359027997265e-01*HX1(0)*HX1(0)
				+ 1.6666666666666665e-01*HX1(0)*HX1(0)*HX1(0)
				+ HX1(0) *HX2(0,1)
				+ 6.9314718055994530e-01*HX2(0,1)
				- HX3(0,0, -1)
				- 2.0000000000000000e+00*HX3(0,0,1)
				- HX3(0,1, -1);
			Hi4(0,1,1,-1) =
				+ 1.1401814106428527e+00
				+ 5.8224052646501250e-01*HX1(0)
				- 3.4657359027997265e-01*HX1(0)*HX1(0)
				- 6.9314718055994530e-01*HX2(0,1);
			Hi4(-1,-1,-1,1) =
				- 5.5504108664821579e-02
				+ 2.4022650695910071e-01*HX1(-1)
				- 3.4657359027997265e-01*HX1(-1)*HX1(-1)
				+ 1.6666666666666666e-01*HX1(-1)*HX1(-1)*HX1(-1)
				- 5.0000000000000000e-01*HX1(-1)*HX1(-1)*HX1(0)
				+ 6.9314718055994530e-01*HX1(-1)*HX1(0)
				+ 5.0000000000000000e-01*HX1(-1)*HX1(0)*HX1(0)
				- 2.4022650695910071e-01*HX1(0)
				- 3.4657359027997265e-01*HX1(0)*HX1(0)
				- 1.6666666666666666e-01*HX1(0)*HX1(0)*HX1(0);
			Hi4(-1,-1,1,1) =
				- 2.4532465311320902e+00
				+ 1.8851605738073271e+00*HX1(-1)
				+ 5.0000000000000000e-01*HX1(-1)*HX1(-1)*HX1(0)
				- 5.0000000000000000e-01*HX1(-1)*HX1(0)*HX1(0)
				- HX1( -1)*HX2(0,-1)
				- HX1( -1)*HX2(0,1)
				- 1.8851605738073271e+00*HX1(0)
				+ 1.6666666666666666e-01*HX1(0)*HX1(0)*HX1(0)
				+ HX3( -1,-1,1)
				+ HX3(0, -1,-1)
				+ HX3(0,0, -1)
				+ HX3(0,0,1)
				+ HX3(0,1, -1);
			Hi4(-1,1,1,1) =
				- 5.5504108664821579e-02
				- 1.6449340668482264e+00*HX1(-1)
				+ 5.0000000000000000e-01*HX1(-1)*HX1(0)*HX1(0)
				+ 1.6449340668482264e+00*HX1(0)
				- 1.6666666666666666e-01*HX1(0)*HX1(0)*HX1(0)
				+ HX1(0) *HX2(-1,1)
				- HX1(0) *HX2(0,-1)
				- HX1(0) *HX2(0,1)
				+ HX3( -1,1,1)
				- HX3(0, -1,1)
				+ HX3(0,0, -1)
				+ HX3(0,0,1)
				- HX3(0,1,1);
		} // nw > 3 endif
	} // (n1,n2) = (-1,1) -- completion endif
	return;
}


/**
 *
 * @brief evaluates the irreducible HPL for y =1
 * it is guaranteed that nw is in the range 2:4, and that (n1,n2)
 * take one of the pairs of values (0,1), (-1,0) or (-1,1)
 */
void hpl::details::fillirr1dhplin1(double y, int nw,
		FA1dD11& HY1, FA2dD11& HY2, FA3dD11& HY3, FA4dD11& HY4,
		int n1, int n2)
{
	// (n1,n2) = (0,1) or (-1,1)
	if (  ( (n1==0) && (n2==1) ) || ( (n1==-1) && (n2==1) ) )
	{
		HY2(0,1) = 1.6449340668482264e+00;
		if (nw>2)
		{
			HY3(0,0,1) = 1.2020569031595942e+00;
			HY3(0,1,1) = 1.2020569031595942e+00;
		}
		if (nw>3)
		{
			HY4(0,0,0,1) = 1.0823232337111381e+00;
			HY4(0,0,1,1) = 2.7058080842778454e-01;
			HY4(0,1,1,1) = 1.0823232337111381e+00;
		}
	} // (n1,n2) = (0,1) or (-1,1) endif

	//************
	// (n1,n2) = (-1,0) or (-1,1)
	if (    ( (n1==-1) && (n2==0) ) || ( (n1==-1) && (n2==1) ) )
	{
		HY2(0,-1) = 8.2246703342411321e-01;
		if (nw>2){
			HY3(0,-1,-1) = 1.5025711289494928e-01;
			HY3(0,0,-1) = 9.0154267736969571e-01;
		}
		if (nw>3){
			HY4(0,-1,-1,-1) = 2.3752366322618485e-02;
			HY4(0,0,-1,-1) = 8.7785671568655302e-02;
			HY4(0,0,0,-1) = 9.4703282949724591e-01;
		}
	} // (n1,n2) = (-1,0) or (-1,1) endif
	// (n1,n2) = (-1,1) -- completion
	if ( (n1==-1) && (n2==1) )
	{
		HY2(-1,1) = + 5.8224052646501250e-01;
		if (nw>2) {
			HY3(0,-1,1) = + 2.4307035167006157e-01;
			HY3(0,1,-1) = + 5.0821521280468485e-01;
			HY3(-1,-1,1) = + 9.4753004230127705e-02;
			HY3(-1,1,1) = + 5.3721319360804020e-01;
		}
		if (nw>3){
			HY4(0,0,-1,1) = + 1.1787599965050932e-01;
			HY4(0,0,1,-1) = + 1.7284527823898438e-01;
			HY4(0,-1,0,1) = + 2.0293560632083841e-01;
			HY4(0,-1,-1,1) = + 3.4159126166513913e-02;
			HY4(0,-1,1,-1) = + 5.4653052738263652e-02;
			HY4(0,1,-1,-1) = + 1.1412342741606084e-01;
			HY4(0,-1,1,1) = + 9.3097125991768577e-02;
			HY4(0,1,-1,1) = + 1.9355535381306524e-01;
			HY4(0,1,1,-1) = + 4.3369237704895519e-01;
			HY4(-1,-1,-1,1) = + 1.4134237214990008e-02;
			HY4(-1,-1,1,1) = + 4.0758239159309251e-02;
			HY4(-1,1,1,1) = + 5.1747906167389938e-01;
		}
	} // (n1,n2) = (-1,1) -- completion endif
	return;
}
